from django.contrib import admin
from guardian.admin import GuardedModelAdmin
from .models import *
#import/export module
from import_export.admin import ImportExportModelAdmin

from modeltranslation.admin import TabbedTranslationAdmin
#for representative upload
from django import forms
import csv
from csv import DictReader
from django.urls import path
from django.shortcuts import redirect,render

from risefor_lobbying.functions.uploadRepresentatives import *
from risefor_lobbying.functions.updateApprovedEmails import *

import os

from reversion.admin import VersionAdmin
##for TinyMCE
from django import forms
from tinymce.widgets import TinyMCE

##########
#Representative -> Action group linking
##########
# def add_representative_to_action(modeladmin, request, queryset):
#     # do something with the queryset
#     print("hey")
# add_representative_to_action.short_description = 'Cibler '

##########
#Inline models
##########
class PhoneInline(admin.TabularInline):
    model = Phone
    extra = 1

class EmailInline(admin.TabularInline):
    model = Email
    extra = 1

class TwitterInline(admin.TabularInline):
    model = TwitterAccount
    extra = 1

class actionStepInline(admin.TabularInline):
    exclude = ('urlid','is_backlink','allow_create_backlink')
    model = ActionStep
    extra = 1

class instagramMessageInline(admin.TabularInline):
    exclude = ('urlid','is_backlink','allow_create_backlink')
    search_fields = ['name']
    model = InstagramMessage
    extra = 1

class twitterMessageInline(admin.TabularInline):
    exclude = ('urlid','is_backlink','allow_create_backlink')
    search_fields = ['name']
    model = TwitterMessage
    extra = 1

class OrganisationLogoInline(admin.TabularInline):
    exclude = ('urlid','is_backlink','allow_create_backlink')
    model = OrganisationLogoForAction
    extra = 1

class imageToShareInline(admin.TabularInline):
    exclude = ('urlid','is_backlink','allow_create_backlink')
    model = imageToShare
    extra = 1



##########
#Model lists & Pages
##########

@admin.register(ApprovedEmail)
class ApprovedEmailsAdmin(admin.ModelAdmin):
    search_fields = ['name'] 
    change_list_template = './admin/entities/update-approved-emails.html'

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            
            path('update-approved-emails/', self.update_approved_emails),
        ]
        return my_urls + urls
    def update_approved_emails(self,request):
        print("let's update the list of approved emails")
        updateApprovedEmails(self,request)

        return render(
            request, "./admin/upload-forms/start-update-emails.html"
        )


@admin.register(representativeFunction)
class representativeFunctionAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(representativeContactPlateform)
class representativeContactPlateformAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(missionParlementaire)
class missionParlementaireAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(representativeFinishedFunction)
class representativeFinishedFunctionAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(additionalFunction)
class additionalFunctionAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(officeParlementaire)
class officeParlementaireAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(assembleeParlementaireInternationale)
class assembleeParlementaireInternationaleAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(delegationAssemblee)
class delegationAssembleeAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(missionInformation)
class missionInformationAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(groupePolitiqueParlement)
class groupePolitiqueParlementAdmin(admin.ModelAdmin):
    search_fields = ["name"]

@admin.register(Circonscription)
class CirconscriptionAdmin(admin.ModelAdmin):
    search_fields = ["name"]
    list_filter = ["name"]



#Representative PAGE & List
@admin.register(Representative)
class RepresentativeAdmin(ImportExportModelAdmin,VersionAdmin):
    ###Display & filters
    inlines = [ PhoneInline, EmailInline,TwitterInline]
    list_filter = ('function','politicalParty','permanentCommission','election_department','studyGroups')
    # dropdown_filter = ('election_department') #needs specific package to enable
    list_display = ['fullname','function']
    search_fields = ['fullname']
    autocomplete_fields = ["politicalParty","parliamentaryMissions","missionInformations","delegationAssemblees","assembleeParlementaireInternationales","officeParlementaires","additionalFunctions","finishedFunctions"]
    # 

    # actions = [add_representative_to_action, ]

    ###csv upload function
    # change_list_template = "./admin/entities/upload-representatives-changelist.html"

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            path('import-csv/', self.import_csv),
        ]
        return my_urls + urls

    def import_csv(self, request):
        if request.method == "POST":
            csv_file = request.FILES["csv_file"]
            decoded_file = csv_file.read().decode('utf-8').splitlines()
            reader = DictReader(decoded_file)
            print(reader)
            # Create Hero objects from passed in data
            # ...
            uploadRepresentatives(reader)
            # for representative in reader:
            #     i = 0
            #     print(representative["FULL NAME"]) 
            
            self.message_user(request, "Les élues ont bien été importé")
            logger.notice('We will now update the Approved emails model')
            updateApprovedEmails(self,request)
            return redirect("..")
        form = CsvImportForm()
        payload = {"form": form}
        return render(
            request, "./admin/upload-forms/representative-upload-csv.html", payload
        )


#####
## Admin model views
#####
#URL Shortner
@admin.register(UrlShortner)
class urlShortnerAdmin(GuardedModelAdmin):
    exclude = ('urlid','is_backlink','allow_create_backlink')
    list_display = ['actionName']
    readonly_fields=['actionName']

#Action steps
@admin.register(ActionStep)
class ActionStepAdmin(GuardedModelAdmin):
    exclude = ('urlid','is_backlink','allow_create_backlink')
    list_display = ['name']

@admin.register(commissionPermanente)
class commissionPermanenteAdmin(GuardedModelAdmin):
    exclude = ('urlid','is_backlink','allow_create_backlink')
    search_fields = ['name']



#ACTION GROUP PAGE & List
@admin.register(ConsultationPublic)
class consultationAdmin(GuardedModelAdmin):
    exclude = ('slug',)
    list_display = ['name','status']
    readonly_fields=('urlid','author','instanceName','instanceHostName')

class AppelSoutienAdminForm(forms.ModelForm):
    class Meta:
        model = AppelSoutien
        fields = '__all__'
        widgets = {
            'objective': TinyMCE(attrs={'cols': 80, 'rows': 20}),
            'description': TinyMCE(attrs={'cols': 80, 'rows': 20}),
        }

@admin.register(AppelSoutien)
class appelSoutienAdmin(VersionAdmin):
    form = AppelSoutienAdminForm
    # list_display = ['name','status']
    exclude = ('is_backlink','allow_create_backlink')
    readonly_fields=('urlid',)
    fieldsets = ( 
        ("Statistiques de l'appel", { 'fields': ('signUpCounter',)}),
        ("Informations de l'appel", { 'fields': ('name','status','objective','description','image','mobileImage','shortLink')}),
        ("Email de confirmation envoyé aux signataires", { 'fields': ('emailConfirmationSujet','emailConfirmationContenu','emailConfirmationReplyTo',)}),
    )

#ACTION GROUP PAGE & List
class ActionGroupAdminForm(forms.ModelForm):
    class Meta:
        model = ActionGroup
        fields = '__all__'
        widgets = {
            'description': TinyMCE(attrs={'cols': 80, 'rows': 20}),
        }
    # def clean_description(self):
    #     content = self.cleaned_data.get('description', '')
    #     # Remove <br> tags
    #     cleaned_content = content.replace('<br>', '').replace('<br />', '')
    #     logger.debug(f"cleaned content is {cleaned_content}")
    #     return cleaned_content



@admin.register(ActionGroup)
class actionGroupAdmin(VersionAdmin):
    form = ActionGroupAdminForm
    inlines = [ instagramMessageInline,twitterMessageInline,actionStepInline,OrganisationLogoInline,imageToShareInline]
    exclude = ('country','urlid','members','actiongroups','conversation','slug','displayContactsByPlatform')
    list_display = ['name','status']
    readonly_fields=('urlid','author','instanceName','instanceHostName','totalContacts','representativeEmailSentCounter','instagramClickCounter','twitterClickCounter','peopleTargettedCounter')
    search_fields = ['name',] 
    # filter_horizontal = ('specificTargetedRepresentatives',)
    autocomplete_fields = ['targetCommission','specificTargetedRepresentatives','targetedOfficials','contactPlateforms']
    # fields=('name','test','image') #here to set order of elements in admin
    fieldsets = ( 
        ("Informations de votre instance Risefor (données venant du fichier settings.py)", { 'fields': ('instanceName','instanceHostName')}),
        ("Statistiques de l'action", { 'fields': ('totalContacts','representativeEmailSentCounter','twitterClickCounter','instagramClickCounter','peopleTargettedCounter','displaydate')}),
        ("Informations générales de votre action", { 'fields': ('author','name','status','objective','description','endactiondate','shortLink')}),
        ("Images bannière",{ 'fields': ('image','mobileImage')}),
        ("Informations supplémentaires", { 'fields': ('titleOrganisationLogoSection', 'emailOrganisation','linkOrganisation','actioninfolink')}), 
        ("Action Physique : Localisation", { 'fields': ('address', 'city','postcode','lat','lng')}), 
        ("Choix de ciblage", { 'fields': ('typeOfTargetting','targetedOfficials','targetCommission','contactPlateforms','addCCToEmail','addPJToEmail')}), 
        ("Contenu pour les Députées", { 'fields': ('electedofficialEmailSubject', 'electedofficialEmailText','TweetMain','Tweet2','Tweet3','Tweet4','representativeCallObjective','representativeCallArgumentation')}), 
        ("Contenu pour les Sénateurices", { 'fields': ('senatorEmailSubject', 'senatorEmailText','senatorTweetMain','senatorTweet2','senatorTweet3','senatorTweet4','senatorCallObjective','senatorCallArgumentation')}),
        ("Contenu pour les Candidates", { 'fields': ('candidateEmailSubject', 'candidateEmailText','candidateTweetMain','candidateTweet2','candidateTweet3','candidateTweet4','candidateCallObjective','candidateCallArgumentation')}),
        ("Ciblage de décisionnnaires précis", { 'fields': ('specificTargetedRepresentatives', 'targetedRepresentativesTitle','targetedEmailSubject','targetedEmailText','targetedTweetMain','targetedTweet2','targetedTweet3','targetedTweet4','targetedCallObjective','targetedCallArgumentation')}),
        ("Communiqué de presse", { 'fields': ('pressreleasetitle', 'pressreleasecontent')}), 
        ("Afficher une vidéo", { 'fields': ('videotitle', 'videourl',)}), 
        ("Afficher une carte des mobilisations", { 'fields': ('mapSignUpSectionTitle', 'mapSignUp',)}), 
        ("Afficher des étapes d'actions", { 'fields': ('actionStepTitle',)}), 
    )
    if settings.PLUGIN_LAW_TRACKER :
        logger.debug(f"Law tracker active, add field to ActionGroup")
        autocomplete_fields += ['targetLaw']
        for fieldset in fieldsets:
            if fieldset[0] == "Choix de ciblage":
                fieldset[1]['fields'] += ('targetLaw',)
    
    class Media:
        js = ('./js/admin-fields.js','./js/admin_tinymce_custom.js')


# Emails sent to representatives
@admin.register(ElectedOfficialContact)
class ElectedOfficialContactAdmin(ImportExportModelAdmin,GuardedModelAdmin):
    list_display= ['userName','created_at',]
    list_filter = ['linkedAction',]


# Emails sent to representatives
@admin.register(TextSupporter)
class TextSupporterAdmin(ImportExportModelAdmin,GuardedModelAdmin):
    list_display= ['fullname','created_at',]
    list_filter = ['linkedSupport',]

@admin.register(TextSupporterPublic)
class TextSupporterPublicAdmin(ImportExportModelAdmin,GuardedModelAdmin):
    list_display= ['name','created_at',]
    list_filter = ['linkedSupport',]


# Emails sent to representatives
@admin.register(Newsletter)
class NewsletterAdmin(ImportExportModelAdmin,GuardedModelAdmin):
    list_display= ['fullname','created_at',]
    search_fields = ['userEmail']
    list_filter = ['signedUpFrom']



@admin.register(ContactTeam)
class contactTeamAdmin(GuardedModelAdmin):
    list_display= ['subject','status','created_at']


#TRANSLATION FIELDS
class riseforAdmin(TabbedTranslationAdmin):
    group_fieldsets = True
    fieldsets = [
        (u'ActionGroup', {'fields': ('name',)}),
        (u'representativeFunction', {'fields': ('name',)}),
        (u'representativeContactPlateform', {'fields': ('name',)}),
    ]


#Hide flag fields from admin
@admin.register(Countrie)
class CountrieAdmin(admin.ModelAdmin):
    exclude = ('flag','flag_css')
    #pass


##########
# Upload representatives from csv
##########
class CsvImportForm(forms.Form):
    csv_file = forms.FileField()

# # @admin.register(ActionGroup)
# class uploadRepresentativeAdmin(admin.ModelAdmin):
    
