"""risefor lobbying URL Configuration"""

from django.contrib.auth import get_user_model
from risefor_lobbying.views import *
from django.urls import path,re_path
from django.views.generic.base import TemplateView
from .views import *
from django.contrib.auth import views as auth_views
from django.contrib.sitemaps.views import sitemap
from .sitemaps import *
from django.conf import settings


User = get_user_model()

sitemaps = {
    'actions': ActionGroupSitemap,
    'representative':RepresentativeSitemap,
    'appel_soutiens':AppelSoutienSitemap,
    'enquete_public':ConsultationPublicSitemap,
}


sitemap_patterns =[
    path('actions.xml', sitemap, {'sitemaps': {'actions': ActionGroupSitemap}}, name='actions_sitemap'),
    path('representative.xml', sitemap, {'sitemaps': {'representative': RepresentativeSitemap}}, name='representative_sitemap'),
    path('appel_soutiens.xml', sitemap, {'sitemaps': {'appel_soutiens': AppelSoutienSitemap}}, name='appel_soutiens_sitemap'),
    path('enquete_public.xml', sitemap, {'sitemaps': {'enquete_public': AppelSoutienSitemap}}, name='enquete_public_sitemap'),
    path('sitemap_index.xml', sitemap, {'sitemaps': {'index': SitemapIndex()}}, name='sitemap_index'),    
]
if settings.PLUGIN_LAW_TRACKER:
    from law_tracker.sitemaps import LawTrackerSitemap
    sitemaps['law-tracker'] = LawTrackerSitemap
    sitemap_patterns.append(
        path('law_tracker.xml', sitemap, {'sitemaps': {'law_tracker': LawTrackerSitemap}}, name='law_tracker_sitemap'),
    )



urlpatterns = [
    re_path(r'^related-officials/', RelatedRepresentativeViewSet.urls()),
    re_path(r'^active-campaigns/', financeCampaignViewSet.urls()),
    re_path(r'^active-actions/', ActionViewSet.urls()),
    re_path(r'^active-appels-soutiens/', AppelSoutienViewSet.urls()),
    re_path(r'^active-consultations/', ConsultationViewSet.urls()),
    re_path(r'^passed-actions/', passedActionViewSet.urls()),
    re_path(r'^passed-consultations/', passedConsultationViewSet.urls()),
    re_path(r'total-actions/', TotalSubmission.urls()),
    re_path(r'^search-representative/', RepresentativeSearchViewSet.urls()),
    path("robots.txt",TemplateView.as_view(template_name="robots.txt", content_type="text/plain")),
    path('email-sent-counter/', increment_email_sent_counter, name='increment_email_sent_counter'),
    path('twitter-click-counter/', increment_twitter_click_counter, name='increment_twitter_click_counter'),
    path('instagram-click-counter/', increment_instagram_click_counter, name='increment_instagram_click_counter'),
    path("s/<str:slugs>", shortLinkRedirect, name="redirect"),
    path('admin/password_reset/', auth_views.PasswordResetView.as_view(), name='admin_password_reset'),
    path('admin/password_reset/done/', auth_views.PasswordResetDoneView.as_view(), name='password_reset_done'),
    path('reset/<uidb64>/<token>/', auth_views.PasswordResetConfirmView.as_view(), name='password_reset_confirm'),
    path('reset/done/', auth_views.PasswordResetCompleteView.as_view(), name='password_reset_complete'),
]
## Protected views
urlpatterns.extend(
    [
        re_path(r"^newsletters/", newsletterViewSet.urls()),
        re_path(r"^electedofficialcontacts/", ElectedOfficialContactViewSet.urls()),
        re_path(r"^textsupporters/", TextSupporterViewSet.urls()),
        re_path(r"^organisationsupporters/", OrganisationSupporterViewSet.urls()),
        re_path(r"^approvedemails/", OrganisationSupporterViewSet.urls()),
        re_path(r"^users/", UsersViewSet.urls()),
    ]
)

urlpatterns.extend(sitemap_patterns)