############
 # Script to upload representatives from a csv
 # Called from import_elected.py & upload-representatives-changelists (admin)
############
import os
from csv import DictReader
from risefor_lobbying.models import *
from django.db.models import Value
from django.conf import settings

from server.logging_config import setup_logging,logging
import logging
import time

import requests # to get image from the web
import shutil # to save it locally                  
import risefor_lobbying
# import argparse

DEBUG = False
sleepAfterCreate = 0

logger = setup_logging(env('LOG_LEVEL'))
#########
## Arg parsing
#########

# Parse command-line arguments
# parser = argparse.ArgumentParser()
# parser.add_argument("--dev", action="store_true", help="Send to real recipient list")



def criticalError(message):
    logger.critical(f"{message}")
    raise ValueError(f"{message}")

def uploadRepresentatives(representattiveDict):
    i = 0
    for row in representattiveDict:
        full_name = row.get("nom_complet") or None
        logger.debug(f"non complet c'est {full_name}")
        if full_name == None:
            firstname = row.get("prénom") or row.get("prenom")
            lastname = row.get("nom")
            full_name = firstname + " " + lastname 
            logger.debug(f"No full name, using first name {firstname} and last {lastname}, full name is : {full_name}")

        logger.info(f"Starting n°{i+1} {full_name}")
        # already_created = Representative.objects.filter(firstname__iexact=row["PRENOM"],
        #                                                 lastname__iexact=row["NOM"]).exists()
        already_created = Representative.objects.filter(fullname__iexact=full_name).exists()

        # try:
        #     already_created = Representative.objects.filter(fullname__iexact=row["FULL NAME"]).exists()
        # except KeyError:
        
        if already_created:
            logger.warning(f"L'élue {full_name} existe, passage en mode mise à jour")
            # representative = Representative.objects.filter(fullname__iexact=row["FULL NAME"]).get()
            representative = Representative.objects.filter(fullname__iexact=full_name).get()
            # print("representative is",representative)
        
        if not already_created:
            logger.warning(f"L'élue {full_name} n'existe pas et va être créé")
            representative = Representative()
        

        #############
        # Global fields
        #############
        representative.uid = row.get("id") or "not set"
        representative.civility = row.get("civilite") or row.get("civ")
        logger.debug(f"civilité is {representative.civility}")

        ## If file provides first/last name, use that
        ## orelse, default back to scraper version will full name served right away
        try :
            representative.firstname = firstname
            representative.lastname = lastname
            representative.fullname = firstname + " " + lastname
            logger.debug(f"saving full name as {representative.fullname}")
        except UnboundLocalError:
            representative.fullname = row.get("nom_complet")

            if representative.fullname == None:
                error_message = f"n°{i+1} n'a pas de nom"
                criticalError(error_message)    
        
        logger.verbose(f"Name saved is {representative.fullname}")
        # title #TODO a ajouter dans le fichier, == titre global, ie président assemblée
        
        # slug
        representative.election_department = row.get("nom_deptmt") or  row.get("departementNom") 
        if representative.election_department == None:
            error_message = f"n°{i+1} - {full_name} | Nom de departement non fournie, assurez vous que la colonne soit bien présente et nommé 'nom_deptmt' ou 'nom_deptmt'"
            criticalError(error_message)
        logger.debug(f"Departement {representative.election_department}")

        #add num département?
        representative.election_region  = row.get("nom_region") or ""
        #rename address en 'adresse permanence'
        representative.address1  = row.get("adresse_permanence") or ""
        representative.postal_code = row.get("cp_permanence") or ""
        representative.city = row.get("ville_permanence") or ""
        
        num_circo = row.get("num_circo") or row.get("circo")
        if num_circo:
            representative.circonscription = representative.election_department + " n°" + num_circo
            logger.debug(f"Circo ajouté {representative.circonscription}")

        representative.allPostalCodes  = row.get("Codes_postaux") or ""
        
        if representative.allPostalCodes == "":
            logger.error("There are no postal codes, all filtering via CP will not work")

        # Handle Country
        pays = row.get("pays") or "FR"
        #get the country model
        countries = Countrie()                        
        countryCheck = Countrie.objects.filter(name__iexact=pays).exists()
        existingCountry = Countrie.objects.all()

        logger.debug(f"Country is {pays} and check result is {countryCheck}")

        if countryCheck is True:
            representative.representativeCountry = existingCountry.get(name__iexact=pays)
        else:
            countries.name = row.get("pays") or "FR"
            countries.country = row.get("pays") or "FR"
            countries.save()
            representative.representativeCountry = existingCountry.get(name__iexact=countries.name)
        
        representative.save()
        logger.verbose(f"first save for {representative}")
        group_politique = row.get("groupe") or row.get("groupe_politique") or ""
        # representative.groupe_sigle = row.get("groupe_sigle")  or ""
        political_party = groupePolitiqueParlement()
        logger.debug(f"political party {political_party}")
        political_party_match = groupePolitiqueParlement.objects.filter(name__iexact=group_politique)
        political_party_function = groupePolitiqueParlement.objects.all()
        #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
        if political_party_match.exists() is True:
            logger.debug(f"additional function exists {group_politique}")
            representative.politicalParty.set(political_party_match)
            logger.debug(f"linked function : {representative.politicalParty}")
        else:
            political_party.name = group_politique
            political_party.save()
            representative.politicalParty.add(political_party)
            logger.verbose(f"commission {representative.politicalParty} created and linked")
            
        groupe_sigle = row.get("groupe_sigle") or row.get("groupeAbrev") or ""
        logger.debug(f"party is {groupe_sigle} {representative.politicalParty}")

        #upload image
        image = row.get("photo") or ""
        if image != '' :
            logger.info(f"Going to fetch the photo {image}")
            image_url = row["photo"]
            filename = representative.fullname.replace(" ","-")
            filename = "Representatives/"+filename+".png"
            filename = filename.replace("/","-")
            #print("filename",filename)
            fileurl = filename
            fileDownloadPath = "./media/"+filename
            filename = filename

            # Open the url image, set stream to True, this will return the stream content.
            r = requests.get(image_url, stream = True)

            # Check if the image was retrieved successfully
            if r.status_code == 200:
                # Set decode_content value to True, otherwise the downloaded image file's size will be zero.
                r.raw.decode_content = True
                
                # Open a local file with wb ( write binary ) permission.
                with open(fileDownloadPath,'wb') as f:
                    shutil.copyfileobj(r.raw, f)
                    
                logger.success(f'Image sucessfully Downloaded {filename},{f}')
                logger.info(f"image file url is {fileurl}")
                representative.photo = filename
            else:
                logger.error(f'Image for {full_name} Couldn\'t be retreived')
        else:
            if representative.civility is not '':
                if ("Mme" in representative.civility):
                    representative.photo = "/default/avatar_f.png"
                else:
                    representative.photo = "/default/avatar_h.png"
            else:
                representative.photo = "/default/avatar_h.png"
            logger.info(f"Setting default picture to {representative.photo}")

        #handle function
        currentFunction = row.get("function")
        logger.debug(f"function is {currentFunction}")
        if currentFunction:
            function = representativeFunction()
            functionCheck = representativeFunction.objects.filter(name__iexact=currentFunction).exists()
            existingFunction = representativeFunction.objects.all()
            
            #print("the function is" , row["function"] ,"and",functionCheck)
            if functionCheck is True:
                logger.verbose(f"function {currentFunction} exists")
                representative.function = existingFunction.get(name__iexact=currentFunction)
            else:
                function.name = currentFunction
                function.save()
                logger.verbose(f"function '{currentFunction}' was created")
                representative.function = existingFunction.get(name__iexact=function.name)
        else:
            error_message = f"Erreur sur {full_name} ligne n°{i+1} \nIl n'y a pas de colonne portant le nom 'function'.Veuillez l'ajouter au fichier et recommencer.\nPour rappel les valeurs par default sont 'Député•e', 'Sénateur•ice', 'Candidat•e•s'"
            criticalError(error_message)
        ##########
        # Postes parlementaires (commissions, groupes d'études, etc)
        ##########
        #Commission permanente
        current_commissionPermanente = row.get("commission_permanente") or row.get("commission") or ""
        if current_commissionPermanente != "":
            commission = commissionPermanente()
            commissionCheck = commissionPermanente.objects.filter(name__iexact=current_commissionPermanente).exists()
            existingCommission = commissionPermanente.objects.all()
            
            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
            if commissionCheck is True:
                representative.permanentCommission = existingCommission.get(name__iexact=current_commissionPermanente)
                logger.debug(f"commission {representative.permanentCommission} linked")
            else:
                commission.name = current_commissionPermanente
                commission.save()
                representative.permanentCommission = existingCommission.get(name__iexact=commission.name)
                logger.verbose(f"commission {representative.permanentCommission} created and linked")
        else:
            logger.error(f"Aucune Commission Permanente pour {full_name} ligne n°{i+1}")

        fonction_commission_permanente = row.get("commission_permanente_function") or ""

        if fonction_commission_permanente != '':
            commissionFunction = parliamentaryPosition()
            commissionFunctionCheck = parliamentaryPosition.objects.filter(name__iexact=fonction_commission_permanente).exists()
            existingCommissionFunction = parliamentaryPosition.objects.all()
            
            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
            if commissionFunctionCheck is True:
                representative.permanentCommissionFunction = existingCommissionFunction.get(name__iexact=fonction_commission_permanente)
                logger.debug(f"commission {representative.permanentCommissionFunction} linked")
            else:
                commissionFunction.name = fonction_commission_permanente
                commissionFunction.save()
                representative.permanentCommissionFunction = existingCommissionFunction.get(name__iexact=commissionFunction.name)
                logger.verbose(f"commission {commissionFunction.name} created and linked")

        current_missions_parlementaires = row.get("missions_parlementaires") or ""
        #missions parlementaires
        if current_missions_parlementaires:
            #get the Parliamentary Mission model
            ParliamentaryMission = missionParlementaire()
            #get the values of missions in column
            thisRepresentativeMissions = current_missions_parlementaires.split(";")
            #get all exiting parliamentaryMissions
            existingParliamentaryMissions = missionParlementaire.objects.all()

            #for each new value, check if it exists
            for thisRepresentativeMission in thisRepresentativeMissions:
                ######### 
                # CLEAN NAME && EXTRACT POST IF IT'S THERE
                    #1st, remove post that is between parenthasis (president, vice, etc)
                thisRepresentativeMission = thisRepresentativeMission.split("(")
                thisRepresentativeMission = thisRepresentativeMission[0]  

                ######### 
                # HANDLE CREATION OF NEW OBJECT 
                exists = existingParliamentaryMissions.filter(name__iexact=thisRepresentativeMission).exists()
                
                #if it doesn't exist, 
                if exists is False:
                    # create it
                    ParliamentaryMission.name = thisRepresentativeMission
                    ParliamentaryMission.id = uuid.uuid4().int & (1<<45)-1
                    ParliamentaryMission.urlid = '' 
                    ParliamentaryMission.save()
                
                #add the mission to the representative's model
                representative.parliamentaryMissions.add(missionParlementaire.objects.filter(name__iexact=thisRepresentativeMission)[0])

        #groupe d'éludes
        current_groupe_etude = row.get("groupe_etude") or ""
        if current_groupe_etude:
            #get the Study group model
            studyGroups = groupeEtude()
            #get the values of missions in column
            thisRepresentativeGroups = current_groupe_etude.split(";")
            #for each new value, check if it exists
            for thisRepresentativeGroup in thisRepresentativeGroups: 
                ######### 
                # CLEAN NAME && EXTRACT POST IF IT'S THERE
                    #1st, remove post that is between parenthasis (president, vice, etc)
                thisRepresentativeGroup = thisRepresentativeGroup.split("(")
                thisRepresentativeGroup = thisRepresentativeGroup[0]  

                #print("groupe étude",thisRepresentativeGroup)
                #get all exiting studyGroups
                existingStudyGroups = groupeEtude.objects.all()
                exists = existingStudyGroups.filter(name__iexact=thisRepresentativeGroup).exists()
                #if it doesn't exist, 
                if exists is False:
                    # create it
                    #print("existing",existingStudyGroups.filter(name__iexact=thisRepresentativeGroup))
                    studyGroups.name = thisRepresentativeGroup
                    studyGroups.id = uuid.uuid4().int & (1<<45)-1
                    studyGroups.urlid = studyGroups.name
                    studyGroups.save()
                #add the mission to the representative's model
                representative.studyGroups.add(groupeEtude.objects.filter(name__iexact=thisRepresentativeGroup)[0])
        
        ##############
        # Info de contacts
        #############
        
        #############
        #TODO : CHECK IF EMAIL ALREADY EXISTS, IF SO PASS
        #############
        current_email = row.get("emails") or row.get("email") or row.get("mail") or ""
        logger.debug(f"start email {current_email}")
        if current_email:
            email = Email()
            allEmails = Email.objects.all()
            #get emails related to the representative
            if (";" in current_email) or ("\n" in current_email) or ("|" in current_email) :
                if ("\n" in current_email):
                    emails = current_email.split("\n")
                elif ("|" in current_email):
                    emails = current_email.split("|")
                else:
                    emails = current_email.split(";")
            else:
                emails = [current_email]
                logger.debug(f"email is string, convert to array to avoid dupplicate code")
            
            for newEmail in emails:
                newEmail = newEmail.replace(" ","")
                emailCheck = Email.objects.filter(name__iexact=newEmail).exists()
                
                if emailCheck is False:
                    # print("creating email")
                    email.name = newEmail.lower()
                    email.representative = representative
                    logger.debug(f"adding email {email.name} to {email.representative}")
                    email.save()
                    logger.verbose(f"New email {newEmail.lower()} was created")
                else:
                    ### Most likely a bug here if we have multiple targets using the same email
                    logger.verbose(f"Email {newEmail.lower()} already exists")
        
        twitterAccounts = row.get("twitter") or ""
        if twitterAccounts:
            newTwitter = twitterAccounts
            newTwitter = newTwitter.replace("[","")
            newTwitter = newTwitter.replace("]","")
            newTwitter = newTwitter.replace("'","")

            twitter = TwitterAccount()

            if newTwitter:
                if (";" in newTwitter) or ("," in newTwitter) or ("|" in newTwitter):
                    if ";" in newTwitter:
                        twitterAccounts = newTwitter.split(";")
                    elif "," in newTwitter:
                        twitterAccounts = newTwitter.split(",")
                    elif "|" in newTwitter:
                        twitterAccounts = newTwitter.split("|")
                else :
                    twitterAccounts = [newTwitter]

                for newTwitter in twitterAccounts:
                    newTwitter = newTwitter.replace("@","")
                    newTwitter = newTwitter.replace("https://x.com","")
                    newTwitter = newTwitter.replace("http://x.com","")
                    newTwitter = newTwitter.replace("x.com","")
                    twitterCheck = TwitterAccount.objects.filter(name__iexact=newTwitter).exists()
                    logger.debug(f"check for {newTwitter} is {twitterCheck}")
                    if twitterCheck is False:
                        twitter.name = newTwitter
                        twitter.representative = representative
                        twitter.save()
                        logger.verbose(f"twitter account {newTwitter} was created for {full_name}")
                    else:
                        logger.verbose(f"twitter account {newTwitter} for {full_name} already exists")


        else:
            logger.info(f"no twitter account for {full_name} line {i+1}")

        current_phone_numbers = row.get("tel") or ""
        if current_phone_numbers:
            newPhone = row["tel"]
            newPhone = newPhone.replace("[","")
            newPhone = newPhone.replace("]","")
            newPhone = newPhone.replace("'","")
            if newPhone:
                if (";" in newPhone) or ("," in newPhone):
                    if ";" in newPhone:
                        phones = newPhone.split(";")
                    if "," in newPhone:
                        phones = newPhone.split(",")
            else:
                phones = [newPhone]

            for newPhone in phones:
                newPhone = newPhone.replace(" ","")
                newPhone = newPhone.replace("-","")
                phoneCheck = Phone.objects.filter(name__iexact=newPhone).exists()

                if phoneCheck is False:
                    phone = Phone()
                    phone.name = newPhone
                    phone.representative = representative
                    phone.save()
                    logger.verbose(f"Phone number {newPhone} was created for {full_name}")
                else:
                    logger.verbose(f"Phone number {newPhone}  for {full_name} already exists")

              

        representative.website = row.get("website") or row.get("sites") or ""
        representative.facebook = row.get("facebook")
        profession = row.get("profession") or row.get("job") or ""
        representative.profession = profession.replace("[","").replace("]","")

        logger.debug(f"Site web : {representative.website}, facebook: {representative.facebook}, profession: {representative.profession}")

        representative.save()
        
        logger.notice(f"La fiche n°{i+1} - {full_name} a bien été créé")
        i+= 1
        time.sleep(sleepAfterCreate)
        if DEBUG and i >= 5:
            break
    
    
    logger.success(f'{i} élues ont été importé')




