from risefor_lobbying.models import *
from django.core.management.base import BaseCommand
import requests
from django.conf import settings
import shutil # to save img   
from lxml import html
from reversion import create_revision
from reversion import revisions as reversion

logger = setup_logging('INFO')

class Command(BaseCommand):
    help = 'Imports representatives'

    def add_arguments(self, parser):
        parser.add_argument(
            "--function",
            help="Ne cibler que certaines fonctions",
            choices=['all','depute','senateur','ministre'], 
            default='all'
        )
        parser.add_argument(
            '--force-update',
            action='store_true',
            help='Upload all historical data ?',
        )
        # parser.add_argument("--yes","-y",action="store_true",help="Yes to questions, exept critical")

    # def import_csv()
    def handle(self, *args, **options):
        representatives = Representative.objects.all()
        logger.notice(f"handle")
        count_dep = 0
        count_not_updated = 0
        force_update = False
        if options['force_update']:
            logger.success(f"force update")
            force_update = True
        
        logger.notice(f"force update {force_update}")
        for representative in representatives:
            function = representative.function.name
            uid = representative.uid
            # logger.spam(f"start image for {representative} {function}")
            
            logger.debug(f"current image name {representative.fullname}")
            photo_is_default = False
            if not representative.photo or not representative.photo.name or ('default' in representative.photo.name):
                photo_is_default = True

            # photo_is_default = True if ( (not representative.photo) or ('default' in representative.photo)) else False
            logger.debug(f"is default ? {photo_is_default}")
            if (photo_is_default == True or representative.photo == None) or force_update == True:
                if 'Député' in function:
                    logger.notice(f"{representative.fullname} needs image update {representative.photo}")
                    count_dep += 1
                    logger.debug(f"{representative.fullname} is {function}")
                    image_source_page = f"https://www.assemblee-nationale.fr/dyn/deputes/{uid}"
                    img_xpath = "*//div[@class='acteur-photo-image']//img/@src"
                    response = requests.get(image_source_page)
                    if response.status_code == 200:
                        tree = html.fromstring(response.content)
                        img_src = tree.xpath(img_xpath)
                        logger.debug(f"we found an img {img_src}")
                        if img_src:
                            img_src = img_src[0]
                            image_request = requests.get(img_src, stream = True)

                            if image_request.status_code == 200:
                                logger.debug(f"start requesting image for {representative.fullname}")
                                filename = representative.fullname.replace(" ","-")
                                filename = filename.replace("/","")
                                filename = f"Representatives-{filename}.png"
                                # filename = filename.replace("/","-")
                                #print("filename",filename)
                                file_download_path = "./media/"+filename
                                filename = filename

                                image_request.raw.decode_content = True
                                with open(file_download_path,'wb') as f:
                                    shutil.copyfileobj(image_request.raw, f)

                                representative.photo = filename
                                with reversion.create_revision():
                                    representative.save()
                                    reversion.set_comment("Representative image updated via script")
                                
                                logger.success(f"{count_dep} {representative.fullname} image upated")
                                count_dep += 1
                            else:
                                logger.error(f"{representative.fullname} issue when downloading image")

                    else:
                        logger.critical(f"Error image status : {response.status_code}")
                        # break
            else:
                logger.info(f"{representative.fullname} already has image {representative.photo}")
                count_not_updated += 1
        
        logger.info(f"Not updated : {count_not_updated} \n Deputee updated : {count_dep}\nTotal = {count_not_updated+count_dep}")