import os
from csv import DictReader
from warnings import catch_warnings
from risefor_lobbying.models import Representative, TwitterAccount,Phone, Email,representativeFunction,commissionPermanente,missionParlementaire,groupeEtude,Countrie
from django.core.management.base import BaseCommand
from django.db.models import Value
from django.db.models.functions import Concat
from django.conf import settings

import requests # to get image from the web
import shutil # to save it locally                  
import risefor_lobbying

import uuid



class Command(BaseCommand):
    help = 'Imports representatives'

    def import_csv(self, options, file_name):
        with open(file_name) as csvFile:
            csvReader = DictReader(csvFile)
            i = 0

            # Importing elected from file
            for row in csvReader:
                fullName = row["PRENOM"] + " "+ row["NOM"]
                #already_created = Representative.objects.filter(firstname__iexact=row["PRENOM"],
                                                                #lastname__iexact=row["NOM"]).exists()
                already_created = Representative.objects.filter(fullname__iexact=fullName).exists()
                # try:
                #     already_created = Representative.objects.filter(fullname__iexact=row["FULL NAME"]).exists()
                # except KeyError:
                
                if already_created:
                    print("already exists",fullName,"update mode")
                    # representative = Representative.objects.filter(fullname__iexact=row["FULL NAME"]).get()
                    representative = Representative.objects.filter(fullname__iexact=fullName).get()
                    # print("representative is",representative)
                
                if not already_created:
                    representative = Representative()
                
                
                try:
                    representative.civility = row["civilite"]
                except KeyError:
                    representative.civility = ''
                    pass
                try:
                    representative.firstname = row["PRENOM"].capitalize()
                except KeyError:
                    pass
                try:
                    representative.lastname = row["NOM"].capitalize()
                except KeyError:
                    pass
                if (representative.firstname != None) and (representative.lastname  != None):
                    representative.fullname = representative.firstname + " " + representative.lastname     
                else:
                    representative.fullname  = row["FULL NAME"]
                print("importing", i, representative.fullname)
                #representative.title = row["TITRE"].capitalize()
                #representative.protocol_role = row["PROTOCOLE ROLE"].capitalize()
                #representative.protocol_person = row["PROTOCOLE PERSONNE"].capitalize()
                representative.politicalParty = row["PARTI POLITIQUE"]
                representative.election_department = row["DEPARTEMENT"]
                #representative.election_region = row["REGION ELECTION"].capitalize()
                #representative.phone_organization = row["TEL ORGANISME"]
                #representative.email_organization = row["EMAIL ORGANISME"]
                #representative.address1 = row["ADRESSE 1"]
                #representative.address2 = row["ADRESSE 2"]
                #representative.postal_code = row["CP"]
                #representative.city = row["VILLE"].capitalize()
                #representative.organization_type = row["TYPE"].capitalize()
                #representative.organization_subtype = row["SOUS TYPE"].capitalize()
                
                representative.commune = row["ALL POSTAL CODES"] 
                try:
                    representative.twitter = row["twitter"]
                except KeyError:
                    pass
                #representative.position_campagne = row["Soutien_RIP"].capitalize()
                
                try:
                    if row["photo"] :
                        print("photo",row["photo"])
                        image_url = row["photo"]
                        filename = representative.fullname.replace(" ","-")
                        filename = "/static/img/Representatives/"+filename+".png"
                        #print("filename",filename)
                        fileurl = filename
                        filename = "."+filename
                        

                        # Open the url image, set stream to True, this will return the stream content.
                        r = requests.get(image_url, stream = True)

                        # Check if the image was retrieved successfully
                        if r.status_code == 200:
                            # Set decode_content value to True, otherwise the downloaded image file's size will be zero.
                            r.raw.decode_content = True
                            
                            # Open a local file with wb ( write binary ) permission.
                            with open(filename,'wb') as f:
                                shutil.copyfileobj(r.raw, f)
                                
                            print('Image sucessfully Downloaded: ',filename)
                        else:
                            print('Image Couldn\'t be retreived')
                        
                        representative.photo = fileurl #urllib.request.urlretrieve(row["photo"])
                except KeyError:
                    pass     
                #print(representative.photo)
                    
                #clean ";" seperator to a line break
                #if row["website"]
                try:
                    representative.website = row["website"]
                except KeyError:
                    pass

                representative.circonscription = row["circonscription"]

                try:
                    representative.profession = row["profession"].replace("[","").replace("]","")
                except KeyError:
                    pass
                try:
                    representative.facebook = row["facebook"]
                except KeyError:
                    pass
                #####
                #HANDLE Empty IMAGES
                    #   If sex given, personnalize image
                    #   else add male default
                #####
                if representative.photo is '':
                    if representative.civility is not '':
                        if ("Mme" in representative.civility) or ("Mlle" in representative.civility) or ("Ma" in representative.civility):
                            representative.photo = settings.BASE_URL+"/static/img/avatar_f.png"
                        else:
                            representative.photo = settings.BASE_URL+"/static/img/avatar_h.png"
                    else:
                        representative.photo = settings.BASE_URL+"/static/img/avatar_h.png"

                #IMPORT REPRESENTATIVE Main commission
                try:
                    if row["permanentCommission"]:
                        commission = commissionPermanente()
                        commissionCheck = commissionPermanente.objects.filter(name__iexact=row["permanentCommission"]).exists()
                        existingCommission = commissionPermanente.objects.all()
                        
                        #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                        if commissionCheck is True:
                            representative.permanentCommission = existingCommission.get(name__iexact=row["permanentCommission"])
                        else:
                            commission.name = row["permanentCommission"]
                            commission.save()
                            representative.permanentCommission = existingCommission.get(name__iexact=commission.name)
                except KeyError:
                    pass

                #IMPORT REPRESENTATIVE Function
                try:
                    if row["function"]:
                        function = representativeFunction()
                        functionCheck = representativeFunction.objects.filter(name__iexact=row["function"]).exists()
                        existingFunction = representativeFunction.objects.all()
                        
                        #print("the function is" , row["function"] ,"and",functionCheck)
                        if functionCheck is True:
                            representative.function = existingFunction.get(name__iexact=row["function"])
                        else:
                            function.name = row["function"]
                            function.save()
                            representative.function = existingFunction.get(name__iexact=function.name)
                except KeyError:
                    pass


                #save Representative
                representative.save()
                #IMPORT REPRESENTATIVE Parlementary missions
                try:
                    if row["parliamentaryMissions"]:
                        #get the Parliamentary Mission model
                        ParliamentaryMission = missionParlementaire()
                        #get the values of missions in column
                        thisRepresentativeMissions = row["parliamentaryMissions"].split(";")
                        #get all exiting parliamentaryMissions
                        existingParliamentaryMissions = missionParlementaire.objects.all()

                        #for each new value, check if it exists
                        for thisRepresentativeMission in thisRepresentativeMissions:
                            ######### 
                            # CLEAN NAME && EXTRACT POST IF IT'S THERE
                                #1st, remove post that is between parenthasis (president, vice, etc)
                            thisRepresentativeMission = thisRepresentativeMission.split("(")
                            thisRepresentativeMission = thisRepresentativeMission[0]  

                            ######### 
                            # HANDLE CREATION OF NEW OBJECT 
                            exists = existingParliamentaryMissions.filter(name__iexact=thisRepresentativeMission).exists()
                            
                            #if it doesn't exist, 
                            if exists is False:
                                # create it
                                ParliamentaryMission.name = thisRepresentativeMission
                                ParliamentaryMission.id = uuid.uuid4().int & (1<<45)-1
                                ParliamentaryMission.urlid = '' 
                                ParliamentaryMission.save()
                            
                            #add the mission to the representative's model
                            representative.parliamentaryMissions.add(missionParlementaire.objects.filter(name__iexact=thisRepresentativeMission)[0])
                except KeyError:
                    pass

                #IMPORT REPRESENTATIVE study groups
                try:
                    if row["studyGroups"]:
                        #get the Study group model
                        studyGroups = groupeEtude()
                        #get the values of missions in column
                        thisRepresentativeGroups = row["studyGroups"].split(";")
                        #for each new value, check if it exists
                        for thisRepresentativeGroup in thisRepresentativeGroups: 
                            ######### 
                            # CLEAN NAME && EXTRACT POST IF IT'S THERE
                                #1st, remove post that is between parenthasis (president, vice, etc)
                            thisRepresentativeGroup = thisRepresentativeGroup.split("(")
                            thisRepresentativeGroup = thisRepresentativeGroup[0]  

                            #print("groupe étude",thisRepresentativeGroup)
                            #get all exiting studyGroups
                            existingStudyGroups = groupeEtude.objects.all()
                            exists = existingStudyGroups.filter(name__iexact=thisRepresentativeGroup).exists()
                            #if it doesn't exist, 
                            if exists is False:
                                # create it
                                #print("existing",existingStudyGroups.filter(name__iexact=thisRepresentativeGroup))
                                studyGroups.name = thisRepresentativeGroup
                                studyGroups.id = uuid.uuid4().int & (1<<45)-1
                                studyGroups.urlid = studyGroups.name
                                studyGroups.save()
                            #add the mission to the representative's model
                            representative.studyGroups.add(groupeEtude.objects.filter(name__iexact=thisRepresentativeGroup)[0])
                except KeyError:
                    pass

                #Import country code
                try:
                    if row["country"]:
                        #get the Study group model
                        countries = Countrie()                        
                        countryCheck = Countrie.objects.filter(name__iexact=row["country"]).exists()
                        existingCountry = Countrie.objects.all()
                        
                        if countryCheck is True:
                            representative.representativeCountry = existingCountry.get(name__iexact=row["country"])
                        else:
                            countries.name = row["country"]
                            countries.country = row["country"]
                            countries.save()
                            representative.representativeCountry = existingCountry.get(name__iexact=countries.name)
                        representative.save()             
                except KeyError:
                    pass
                #IMPORT CONTANCT INFO
                
                #TODO
                if row["email"]:
                    if (";" in row["email"]) or ("\n" in row["email"]) :
                        if ("\n" in row["email"]):
                            emails = row["email"].split("\n")
                        else:
                            emails = row["email"].split(";")
                        for newEmail in emails:
                            newEmail = newEmail.replace(" ","")
                            newEmail = newEmail.replace("-","")
                            if newEmail:
                                email = Email()
                                email.name = newEmail
                                email.representative = representative
                                email.save()

                    else:
                        email = Email()
                        email.name = row["email"]
                        email.representative = representative
                        email.save()

                try:
                    if row["twitter"]:
                        newTwitter = row["twitter"]
                        newTwitter = newTwitter.replace("[","")
                        newTwitter = newTwitter.replace("]","")
                        newTwitter = newTwitter.replace("'","")
                        if newTwitter:
                            if (";" in newTwitter) or ("," in newTwitter):
                                if ";" in newTwitter:
                                    twitterAccounts = newTwitter.split(";")
                                if "," in newTwitter:
                                    twitterAccounts = newTwitter.split(",")
                                    
                                for newTwitter in twitterAccounts:
                                    newTwitter = newTwitter.replace(" ","")
                                    newTwitter = newTwitter.replace("-","")
                                    if newTwitter:
                                        twitter = TwitterAccount()
                                        twitter.name = newTwitter
                                        twitter.representative = representative
                                        twitter.save()

                            else:
                                twitter = TwitterAccount()
                                twitter.name = newTwitter
                                twitter.representative = representative
                                twitter.save()
                except KeyError:
                    pass
                #TODO
                #handle multiple inputs
                try:
                    if row["phone"]:
                        newPhone = row["phone"]
                        newPhone = newPhone.replace("[","")
                        newPhone = newPhone.replace("]","")
                        newPhone = newPhone.replace("'","")
                        if newPhone:
                            if (";" in newPhone) or ("," in newPhone):
                                if ";" in newPhone:
                                    phones = newPhone.split(";")
                                if "," in newPhone:
                                    phones = newPhone.split(",")
                                    
                                for newPhone in phones:
                                    newPhone = newPhone.replace(" ","")
                                    newPhone = newPhone.replace("-","")
                                    if newPhone:
                                        phone = Phone()
                                        phone.name = newPhone
                                        phone.representative = representative
                                        phone.save()

                            else:
                                phone = Phone()
                                phone.name = newPhone
                                phone.representative = representative
                                phone.save()
                except KeyError:
                    pass
                
                i = i + 1
                if options['dev'] and i > 10:
                    break
        
            
            
            # Associating assistants with elected
            #TODO
                #handle new format multiple inputs
            for representative in Representative.objects.all():
                if (representative.function == 'Attache parlementaire'):
                    title_array = representative.title.split(' madame ')
                    title_array = title_array[len(title_array) - 1].split(' monsieur ')
                    assistant_name = title_array[len(title_array) - 1]

                    elected_officials = Representative.objects.annotate(
                        search_name=Concat('firstname', Value(' '), 'lastname'))
                    try:
                        elected_official = elected_officials.get(search_name__iexact=assistant_name)
                        elected_official.assistants.add(representative)
                        elected_official.save()
                    except Representative.DoesNotExist:
                        print(assistant_name + ' not found in database')

    def add_arguments(self, parser):
        parser.add_argument(
            '--dev',
            action='store_true',
            help='Dummy data for development',
        )

        parser.add_argument(
            '--filename',
            help='Name of file to upload'
        )
    

    def handle(self, *args, **options):
        
        args = options
        print(f"args foo is {args['dev']}, "+args['filename'])
        filename =  args['filename']
        if filename:
            if 'None' in filename:
                raise ValueError("Use the flag --filename to specify the name of the file to upload")
        
        filename = 'elected-officials/'+filename+'.csv'
        print("filename",filename)
        rise_for_lobbying_path = os.path.abspath(os.path.dirname(risefor_lobbying.__file__))
        file_name = os.path.join(rise_for_lobbying_path,
                                 #'elected-officials/clean-representative-db.csv')
                                 filename)
        self.import_csv(options=options, file_name=file_name)
