import os
from django.core.management.base import BaseCommand
from csv import DictReader
from collections import OrderedDict
from risefor_lobbying.models import *
from django.conf import settings
from server.logging_config import logging
import json
import time
from django_q.tasks import async_task
import re
from reversion import create_revision
from reversion import revisions as reversion
           

DEBUG = True
logger = setup_logging(env('LOG_LEVEL'))


law_tracker = os.path.join(BASE_DIR,'./law_tracker/data/src/an/dep_sen_min_tous_mandats_et_organes/master/')
organes_files = law_tracker+'master_dep_sen_min-organe.json'
with open(organes_files, 'r', encoding='utf-8') as file:
    organe_data_original = json.load(file)

acteur_historique_files = './law_tracker/data/src/an/dossier-legislatifs/master/master_tous-acteur.json'
with open(acteur_historique_files, 'r', encoding='utf-8') as file:
    acteur_historique_data = json.load(file)

organe_historique_files = './law_tracker/data/src/an/dossier-legislatifs/master/master_organe.json'
with open(organe_historique_files, 'r', encoding='utf-8') as file:
    organe_data_historique = json.load(file)

logger.debug(f'{type(acteur_historique_data)}')
# for acteur in acteur_historique_data:
    # logger.debug(f'{type(acteur.get('acte'))}')

    # logger.debug(acteur)
    # logger.debug(acteur.get("acteur").get("uid").get("#text"))
acteur_historique_index = {acteur.get("acteur").get("uid").get("#text"): acteur for acteur in acteur_historique_data}


postalCodeFile = './risefor_lobbying/data/circo-commune-cp.csv'
with open(postalCodeFile, encoding='utf-8') as postalCodeKeys:
    representativeLocalityData = DictReader(postalCodeKeys)
    localityKeys = OrderedDict()
    circoIndex = 0
    for circo in representativeLocalityData:
        circoKey = circo['CléCodePostale']
        postalCodes = circo['Code Postaux']
        departement = circo['Département']
        departement_num = circo['Département_num']
        # print('keys',circoKey)
        localityKeys[circoIndex] = [circoKey,postalCodes,departement,departement_num]
        circoIndex += 1

def critical_error(message):
    logger.critical(f"{message}")
    # instance = {
    #     'name' : 'admin',
    #     'representativeEmails' : ''
    # }
    # async_task('server.tasks.send_error_email_admin',f"Error uploading representative \n{message}",group="upload_error")
    raise ValueError(f"{message}")

def get_departement_name(dept_id):
    logger.spam(f"start dept id search {dept_id}")
    #  998 corresponds to French Southern and Antarctic Lands (Terres australes et antarctiques françaises, TAAF)
    if dept_id == "998":
        return dept_id
    # issues with 01/02/03 etc, not matching as in data its 1/2/3
    if re.match(r'^0[1-9]$', dept_id):
        dept_id = dept_id.replace("0","")
    # Saint-Pierre-et-Miquelon has 2 ids for some reason
    if dept_id == "997" or dept_id == "Saint-Pierre-et-Miquelon":
        dept_id = "975"
    if dept_id == "099":
        dept_id = "Français établis hors de France"
    #return name linked to dept_id
    for index, values in localityKeys.items():
        if dept_id == values[3]:
            dept_name = values[2]
            logger.info(f"dept_id is {dept_id} {dept_name}")
            return dept_name
            
        
    raise ValueError(logger.critical(f"Search Departement {dept_id} name not found"))


def search_organe_name(uid,search_historique_files=False):
    logger.debug(f"Start search for UID: '{uid}'")
    # Use a generator expression to find the matching organe directly
    if search_historique_files == True:
        logger.spam(f"search in historique organe file")
        organe = next(
            (
                organe_data['organe'] for organe_data in organe_data_historique
                if organe_data['organe'].get("uid") == uid and organe_data['organe'].get("libelle", "").strip() != "Non inscrit"
            ),
            None
        )
    else:
        organe = next(
            (
                organe_data['organe'] for organe_data in organe_data_original
                if organe_data['organe'].get("uid") == uid and organe_data['organe'].get("libelle", "").strip() != "Non inscrit"
            ),
            None
        )
    if organe:
        libelle = organe.get('libelle').strip()
        libelleAgrege = organe.get("libelleAbrege")
        logger.debug(f"Match found: {organe.get('libelle')}")

        # if libelle != "Non inscrit" and organe.get("@xsi:type") != 'GroupePolitique_type':
        if libelle != "Non inscrit":
            if libelle == 'Gouvernement':
                libelle = f"{organe.get('libelle')} {organe.get('libelleAbrege')}"
                logger.verbose(f"New libelle {libelle}")
            if organe.get("@xsi:type") == 'OrganeParlementaireInternational':
                libelle = f"Organe Parlementaire International : {libelle}"
                logger.verbose(f"New libelle {libelle}")
                
            return [libelle,libelleAgrege]
        
    else:
        logger.debug(f"No organe for {uid}")
         

def get_previous_legislature_functions(representative,acteur_id):
    logger.debug(f"start searching previous legislature {acteur_id}")
    acteur = acteur_historique_index.get(acteur_id)
    for acteur_label,details in acteur.items():
        mandats = details.get("mandats", {}).get("mandat", [])
        if mandats:
            for mandat in mandats:
                logger.spam(f"mandat is {mandat.get('uid')}")
                post = mandat.get('infosQualite').get('codeQualite') or mandat.get('infosQualite').get('libQualiteSex') or mandat.get('infosQualite').get('libQualite')
                if post is None:
                    logger.spam(f"post is {mandat}")
                    critical_error(post)
                
                if 'non-inscrit' in post.lower():
                    continue
                    
                mandat_uid = mandat.get('uid')
                date_debut = mandat.get('dateDebut')
                date_fin = mandat.get('dateFin')
                organe_ref = mandat.get('organes').get('organeRef')
                if date_fin is not None:
                    if type(organe_ref) == list:
                        organe = []
                        for current_organe in organe_ref:
                            logger.spam(f"current organe before search {current_organe}")
                            name = search_organe_name(current_organe,True)
                            logger.spam(f"after search {name}")
                            organe.append(name[0])

                        logger.spam(f"organe list before str {organe}")
                        organe = ' ,'.join(organe)
                        organe = [organe,'']
                        logger.debug(f"joined names are {organe}")
                        
                    else:
                        organe = search_organe_name(mandat.get('organes').get('organeRef'),True)

                    if (organe is None) :
                        logger.debug(f"post is {mandat.get('organes')} {mandat}")
                        critical_error(organe)

                    #if organe returns None -> it's non-inscrit, no need
                    logger.debug(f"start organe {organe} ")
                    if organe:
                        organe = organe[0]
                        logger.spam(f"organe search result {organe} {mandat.get('organes').get('organeRef')}")
                        current_mandat = f"{post} {organe} {date_debut} {date_fin}"
                        logger.spam(f"finished mandat name {current_mandat} ")
                    
                        passed_function = representativeFinishedFunction()
                        passed_function_match = representativeFinishedFunction.objects.filter(name__iexact=current_mandat)
                        existing_passed_function = representativeFinishedFunction.objects.all()
                        #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                        if passed_function_match.exists() is True:
                            logger.spam(f"Finished Function passed exists {current_mandat} {acteur_id}")
                            passed_function_update = passed_function_match.get()
                            passed_function_update.dateDebut = date_debut
                            passed_function_update.dateFin = date_fin
                            passed_function_update.post = post
                            passed_function_update.organe = organe
                            passed_function_update.name = current_mandat
                            passed_function_update.save()
                            representative.finishedFunctions.add(*passed_function_match)
                            logger.verbose(f"linked function : {representative.finishedFunctions}")
                        else:
                            passed_function.uid = mandat_uid
                            passed_function.dateDebut = date_debut
                            passed_function.dateFin = date_fin
                            passed_function.post = post
                            passed_function.organe = organe
                            passed_function.name = current_mandat
                            passed_function.save()
                            representative.finishedFunctions.add(passed_function)
                            logger.verbose(f"commission {representative.finishedFunctions} created and linked")


def get_passed_function(representative,acteur,uid):
    mandats = acteur.get("mandats", {}).get("mandat", [])
    function = None
    # logger.debug(f"start get function {}")
    if type(mandats) == dict:
        mandats = [mandats]
    for mandat in mandats :
        post = mandat.get('infosQualite').get('codeQualite') or mandat.get('infosQualite').get('libQualiteSex') or mandat.get('infosQualite').get('libQualite')
        if post is None:
            logger.debug(f"post is {mandat}")
            critical_error(post)
        if 'non-inscrit' in post.lower():
            continue
        # if mandat.get("@xsi:type") == 'GroupePolitique_type':
        #     continue
        mandat_uid = mandat.get('uid')
        date_debut = mandat.get('dateDebut')
        date_fin = mandat.get('dateFin')
        if (date_fin is not None) and (mandat.get('organes').get('organeRef') != 'PO840056'):
            organe = search_organe_name(mandat.get('organes').get('organeRef'))
            #if organe returns None -> it's non-inscrit, no need
            logger.debug(f"start passed functions {organe} {uid}")
            if organe:
                organe = organe[0]
                logger.spam(f"organe search result {organe} {mandat.get('organes').get('organeRef')}")
                current_mandat = f"{post} {organe} {date_debut} {date_fin}"
                logger.verbose(f"finished mandat name {current_mandat} ")
            
                passed_function = representativeFinishedFunction()
                passed_function_match = representativeFinishedFunction.objects.filter(name__iexact=current_mandat)
                existing_passed_function = representativeFinishedFunction.objects.all()
                #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                if passed_function_match.exists() is True:
                    logger.spam(f"Finished function exists {current_mandat}")
                    representative.finishedFunctions.add(*passed_function_match)
                    logger.verbose(f"linked function : {representative.finishedFunctions}")
                else:
                    passed_function.uid = mandat_uid
                    passed_function.dateDebut = date_debut
                    passed_function.dateFin = date_fin
                    passed_function.post = post
                    passed_function.organe = organe
                    passed_function.name = current_mandat
                    passed_function.save()
                    representative.finishedFunctions.add(passed_function)
                    logger.verbose(f"commission {representative.finishedFunctions} created and linked")


####
# Set current function
# Take advantage of this to also set previous functions
####
def get_acteur_function(acteur):
    mandats = acteur.get("mandats", {}).get("mandat", [])
    function = None
    # logger.debug(f"start get function {}")
    if type(mandats) == dict:
        mandats = [mandats]

    for mandat in mandats :
        typeOrgane = mandat.get("typeOrgane", {})
        date_fin = mandat.get("dateFin")
        logger.spam(f"date fin is {date_fin}")
        if date_fin is None:
            if typeOrgane == "MINISTERE":
                function = "Ministre"
                return function
            elif typeOrgane == "SENAT":
                function = "Sénateur•ice"
            elif typeOrgane == "ASSEMBLEE":
                function = "Député•e"
            else:
                pass
        else:
            pass
    logger.spam(f"going to return {function}")
    return function

def add_email_address(uid,contact_info,representative):
    email = Email()
    logger.spam(f"start setting email {uid} {contact_info}")
    email_address = contact_info.get("valElec")
    emailCheck = Email.objects.filter(name__iexact=email_address).exists()
    if emailCheck is False:
        # print("creating email")
        email.name = email_address.lower()
        email.representative = representative
        logger.spam(f"adding email {email.name} to {email.representative}")
        email.save()
        logger.spam(f"New email {email_address.lower()} was created")
    else:
        ### Most likely a bug here if we have multiple targets using the same email
        logger.spam(f"Email {email_address.lower()} already exists")

def add_twitter_account(uid,contact_info,representative):
    twitter = TwitterAccount()
    newTwitter = contact_info.get("valElec")
    newTwitter = newTwitter.replace("@","")
    newTwitter = newTwitter.replace("https://x.com","")
    newTwitter = newTwitter.replace("http://x.com","")
    newTwitter = newTwitter.replace("x.com","")
    twitterCheck = TwitterAccount.objects.filter(name__iexact=newTwitter).exists()
    logger.spam(f"check for {newTwitter} is {twitterCheck}")
    if twitterCheck is False:
        twitter.name = newTwitter
        twitter.representative = representative
        twitter.save()
        logger.spam(f"twitter account {newTwitter} was created for {uid}")
    else:
        logger.verbose(f"twitter account {newTwitter} for {uid} already exists")


def add_tel_number(uid,contact_info,representative):
    newPhone = contact_info.get("valElec")
    logger.debug(f"add tel {newPhone}")
    phoneCheck = Phone.objects.filter(number__iexact=newPhone).exists()
    if phoneCheck is False:
        phone = Phone()
        phone.number = newPhone
        phone.representative = representative
        phone.save()
        logger.verbose(f"Phone number {newPhone} was created for {uid}")
    else:
        logger.verbose(f"Phone number {newPhone}  for {uid} already exists")


class Command(BaseCommand):
    help = 'Imports representatives'

    def add_arguments(self, parser):
        parser.add_argument(
            '--historical-data',
            action='store_true',
            help='Upload all historical data ?',
        )
        parser.add_argument("--dev","-y",action="store_true",help="")
        parser.add_argument("--type",
            help="Type de données à mettre à jour.",
            choices=['senateur','deputee','ministre','all'], 
            default='--all'
        )
        parser.add_argument("--sleep",help="Customize time between emails send")

    # def import_csv()
    def handle(self, *args, **options):

        ### Sleep function to when need to update and not enough resource available at the moment (getting killed)
        if options['sleep']:
            logger.info(f"sleep after create for {options['sleep']}secs")
            sleepAfterSend = int(options['sleep'])

        representative_file = law_tracker+'master_dep_sen_min-acteur.json'
        no_save = False
        logger.info(f"Rep file {representative_file}")
        # logger.info(f" {organes_files}")

        with open(representative_file, 'r', encoding='utf-8') as file:
            representative_data = json.load(file)
        
        representative_data = representative_data
        # logger.debug(f"rep data {representative_data}")
        count = 0
        for e in representative_data:
            for actor,details in e.items():

                # logger.debug(f"rep data {details}")
                uid = details.get("uid").get("#text")
                representative_function = get_acteur_function(details)

                if representative_function == None:
                    logger.critical(f"no function for {uid}")
                    representative_function = "Not set"
                    # critical_error(f"no function for {uid}")
                
                #############
                # Stop Upload on :
                    # Senators : until more work is done (commissions, etc)
                    # Not set : Due to not active representative_function
                #############
                if 'not set'  in representative_function.lower():
                    logger.warn(f"{uid} is a {representative_function} - Pass")
                    continue
               
                if options['type'] != 'all':
                    if options['type'] == 'senateur':
                        if 'sénat' not in representative_function.lower():
                            logger.warn(f"{uid} is a {representative_function} - Pass")
                            continue
                    if options['type'] == 'deputee':
                        if 'déput' not in representative_function.lower():
                            logger.warn(f"{uid} is a {representative_function} - Pass")
                            continue
                    if options['type'] == 'ministre':
                        if 'mini' not in representative_function.lower():
                            logger.warn(f"{uid} is a {representative_function} - Pass")
                            continue
                
                logger.spam(f"function is {representative_function} {uid}")

                etat_civil = details.get("etatCivil").get("ident")
                if etat_civil:
                    civilite = etat_civil.get("civ")
                    firstname = etat_civil.get("prenom")
                    lastname = etat_civil.get("nom")
                else:
                    critical_error(f"pas d'etat civil pour {uid}")

                full_name = firstname + " " + lastname

                already_created = Representative.objects.filter(uid__iexact=uid).exists()
                ### during transition, check if there's the exact name; Especially for senateurs that we haven't updated yet
                if not already_created:
                    already_created = Representative.objects.filter(fullname__icontains=full_name).exists()
                    logger.debug(f"{full_name} matched using name  {already_created} ")

                logger.notice(f"{full_name} existed before ? {already_created} ")

                if already_created:
                    logger.debug(f"L'élue {full_name} existe, passage en mode mise à jour")
                    # representative = Representative.objects.filter(fullname__iexact=row["FULL NAME"]).get()
                    try:
                        representative = Representative.objects.filter(uid__iexact=uid).get()
                    except Exception as e:
                        representative = Representative.objects.filter(fullname__icontains=full_name).get()
                    # print("representative is",representative)
                
                if not already_created:
                    logger.debug(f"L'élue {full_name} n'existe pas et va être créé")
                    representative = Representative()
            
                #############
                # Global fields
                #############
                representative.uid = uid
                representative.civility = civilite
                representative.firstname = firstname
                representative.lastname = lastname
                representative.fullname = full_name

                #############
                # Function
                #############
                # representative.function = representative_function
                function = representativeFunction()
                functionCheck = representativeFunction.objects.filter(name__iexact=representative_function).exists()
                existingFunction = representativeFunction.objects.all()
                
                
                #print("the function is" , row["function"] ,"and",functionCheck)
                if functionCheck is True:
                    logger.verbose(f"function {representative_function} exists")
                    representative.function = existingFunction.get(name__iexact=representative_function)
                else:
                    function.name = representative_function
                    function.save()
                    logger.verbose(f"function '{representative_function}' was created")
                    representative.function = existingFunction.get(name__iexact=function.name)
                ####
                ## Fonction - version affichée
                ####
                if representative_function != "not set":
                    if civilite == "Mme" or civilite == "Madame":
                        if representative_function == "Ministre":
                            representative.display_function = "Ministre"
                        elif representative_function == "Sénateur•ice":
                            representative.display_function = "Sénatrice"
                        elif representative_function == "Député•e":
                            representative.display_function = "Députée"
                            ## Save profile page on Parlement with UID
                            representative.pageParlement = f"https://www.assemblee-nationale.fr/dyn/deputes/{uid}"

                    else:
                        if representative_function == "Ministre":
                            representative.display_function = "Ministre"
                        elif representative_function == "Sénateur•ice":
                            representative.display_function = "Sénateur"
                        elif representative_function == "Député•e":
                            representative.display_function = "Député"
                            ## Save profile page on Parlement with UID
                            representative.pageParlement = f"https://www.assemblee-nationale.fr/dyn/deputes/{uid}"

                            

                #############
                # Info personne
                #############
                profession = details.get("profession",{}).get("socProcINSEE",{}).get("catSocPro",{})
                ## When profession not given they answer a dict, example 'Naturel_georges' or 'Margate_marianne'
                if profession and not isinstance(profession, dict):
                    logger.spam(f"la profession est {profession}")
                    representative.profession = profession.replace("  "," ")
                # profession = details.get("",{})
                
                #### 
                ## For creations, 1st save m2m saves in later steps need @id
                ####
                if not already_created:
                    logger.notice(f"Create {uid} before adresse section")
                    representative.save()

                #############
                # Parse through "mandat"
                #############
                section_mandats = details.get("mandats").get("mandat")
                if type(section_mandats) == dict:
                    section_mandats = [section_mandats]
                for mandat in section_mandats:  
                    group_politique = ""
                    if mandat.get("typeOrgane") == None:
                        pass
                    
                    #####
                    ## Group politique
                    ####
                    elif (mandat.get("typeOrgane") == "GP") or (mandat.get("typeOrgane") == "GROUPESENAT") or (mandat.get("typeOrgane") == "PARPOL"):
                        group_politique_uid = mandat.get("organes").get("organeRef")
                        group_politique_search = search_organe_name(group_politique_uid)
                        logger.debug(f"groupe search returned {group_politique_search}")
                        ## dont always update, cause of the "non inscrit" issue
                        if group_politique_search:
                            logger.spam(f"groupe match {group_politique_search}")
                            group_politique = group_politique_search[0]
                            groupe_sigle =group_politique_search[1]
                            

                        if group_politique == None:
                            critical_error(f"pas de group_politique {group_politique_uid} {uid}")
                        
                        #only update if there's content
                        if group_politique != "":
                            logger.spam(f"save groupe : {group_politique}")
                            # representative.politicalParty = group_politique
                            group_politique = group_politique.strip()

                            political_party = groupePolitiqueParlement()
                            political_party_match = groupePolitiqueParlement.objects.filter(name__iexact=group_politique)
                            political_party_function = groupePolitiqueParlement.objects.all()
                            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                            if political_party_match.exists() is True:
                                representative.politicalParty.set(political_party_match)
                                logger.debug(f"linked Political Party: {representative.politicalParty}")
                            else:
                                political_party.name = group_politique.strip()
                                political_party.save()
                                representative.politicalParty.add(political_party)
                                logger.verbose(f"commission {representative.politicalParty} created and linked")

                    #####
                    ## Election location
                    ####
                    elif ((mandat.get("typeOrgane") == "SENAT") and (mandat.get("infosQualite").get("libQualite").lower() == 'senateur') ) or (mandat.get("typeOrgane") == "ASSEMBLEE"):
                        # if (mandat.get("typeOrgane") == "SENAT") and (mandat.get)
                        # function_temp = mandat.get("typeOrgane")
                        election_info = mandat.get("election").get("lieu")
                        election_department = election_info.get("numDepartement")
                        election_region = election_info.get("region")
                        num_circo = election_info.get("numCirco")
                        logger.verbose(f"election location : {election_department}")
                
                    #####
                    ## Commission permantente (COMPER)
                    #####
                    elif ((mandat.get("typeOrgane") == "COMPER")) or (mandat.get("typeOrgane") == "COMSENAT"):
                        # exclude ended posts
                        if mandat.get("dateFin") is None:
                            current_commission = search_organe_name(mandat.get('organes',{}).get('organeRef',{}))
                            logger.debug(f"Commission name {current_commission}")
                            current_commission = current_commission[0]
                            current_commission = current_commission.strip()
                            

                            commission = commissionPermanente()
                            commission_check = commissionPermanente.objects.filter(name__iexact=current_commission).exists()
                            existing_commission = commissionPermanente.objects.all()
                            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                            if commission_check is True:
                                representative.permanentCommission = existing_commission.get(name__iexact=current_commission)
                                logger.verbose(f"commission {representative.permanentCommission} linked")
                            else:
                                commission.name = current_commission
                                commission.save()
                                representative.permanentCommission = existing_commission.get(name__iexact=commission.name)
                                logger.verbose(f"commission {representative.permanentCommission} created and linked")
                    
                    #####
                    ## Fonction additionnels, including :
                        # BUREAU = Bureau de l'assemblée
                        # DELEGBUREAU = 
                        # CONFPT = Conference des presidents (de commission)
                        # COMNL = Commission Nationale
                        # COMSPSENAT = Commission spéciale au Sénat
                        # CNPE : Conseil National de la Protection de l’Enfance
                        # CNPS : COmmission spécial AN
                        # CMP
                        # ORGEXTPARL : Organisme extra-parlementaire
                    #####
                    
                    elif  (mandat.get("typeOrgane") == "CMP") or (mandat.get("typeOrgane") == "BUREAU") or (mandat.get("typeOrgane") == "ORGEXTPARL")  or (mandat.get("typeOrgane") =="DELEGBUREAU") or  (mandat.get("typeOrgane") == "CONFPT")  or  (mandat.get("typeOrgane") == "CNPS")  or (mandat.get("typeOrgane") == "COMNL") or (mandat.get("typeOrgane") == "COMSPSENAT")or (mandat.get("typeOrgane") == "CNPE"):
                        # add_m2m_field(current_object,target_model,current_key,target_key)
                        logger.debug(f"mandat {mandat.get('typeOrgane')} {uid}")
                        if mandat.get("dateFin") is None:
                            current_mandat = search_organe_name(mandat.get('organes',{}).get('organeRef',{}))
                            current_mandat = current_mandat[0].strip()
                            logger.spam(f"Commission name {current_mandat}")

                            additional_function = additionalFunction()
                            additional_function_match = additionalFunction.objects.filter(name__iexact=current_mandat)
                            existing_additional_function = additionalFunction.objects.all()
                            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                            if additional_function_match.exists() is True:
                                logger.spam(f"additional function exists {current_mandat}")
                                representative.additionalFunctions.add(*additional_function_match)
                                logger.verbose(f"linked function : {representative.additionalFunctions}")
                            else:
                                additional_function.name = current_mandat
                                additional_function.save()
                                representative.additionalFunctions.add(additional_function)
                                logger.verbose(f"commission {representative.additionalFunctions} created and linked")

                    #GE -> Groupe d'etude en attendant de savoir ou le mettre
                    elif (mandat.get("typeOrgane") == "GE"):
                        groupe_etude = groupeEtude()
                        current_mandat = search_organe_name(mandat.get('organes',{}).get('organeRef',{}))
                        current_mandat = current_mandat[0].strip()
                        logger.spam(f"Groupe etude name {current_mandat}")
                        groupe_etude_match = groupeEtude.objects.filter(name__iexact=current_mandat)

                        #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                        if groupe_etude_match.exists() is True:
                            logger.spam(f"Groupe etude exists {current_mandat}")
                            representative.studyGroups.add(*groupe_etude_match)
                            logger.verbose(f"linked Groupe etude : {representative.studyGroups}")
                        else:
                            groupe_etude.name = current_mandat
                            groupe_etude.save()
                            representative.studyGroups.add(groupe_etude)
                            logger.verbose(f"Groupe etude {representative.studyGroups} created and linked")
                    ## TODO : Create models M2M for jobs below 
                
                    # OFFPAR = Office parlementaire 
                    elif (mandat.get("typeOrgane") == "OFFPAR"):
                        # add_m2m_field(current_object,target_model,current_key,target_key)
                        logger.spam(f"mandat {mandat.get('typeOrgane')} {uid}")
                        if mandat.get("dateFin") is None:
                            current_mandat = search_organe_name(mandat.get('organes',{}).get('organeRef',{}))
                            current_mandat = current_mandat[0].strip()
                            logger.debug(f"Office parlementaire name {current_mandat}")

                            additional_function = officeParlementaire()
                            additional_function_match = officeParlementaire.objects.filter(name__iexact=current_mandat)
                            existing_additional_function = officeParlementaire.objects.all()
                            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                            if additional_function_match.exists() is True:
                                logger.spam(f"Office parlementaire exists {current_mandat}")
                                representative.officeParlementaires.add(*additional_function_match)
                                logger.verbose(f"linked function : {representative.officeParlementaires}")
                            else:
                                additional_function.name = current_mandat
                                additional_function.save()
                                representative.officeParlementaires.add(additional_function)
                                logger.verbose(f"commission {representative.officeParlementaires} created and linked")

                    # API =>  French parliamentary system, the code "API" (Assemblée Parlementaire Internationale)
                    # GEVI --> Groupe Etude a vocation Internationals
                    # GA
                    elif (mandat.get("typeOrgane") == "GA") or (mandat.get("typeOrgane") == "API") or (mandat.get("typeOrgane") == "GEVI") :
                        # add_m2m_field(current_object,target_model,current_key,target_key)
                        logger.spam(f"mandat {mandat.get('typeOrgane')} {uid}")
                        if mandat.get("dateFin") is None:
                            current_mandat = search_organe_name(mandat.get('organes',{}).get('organeRef',{}))
                            current_mandat = current_mandat[0].strip()
                            logger.spam(f"AP Internationale name {current_mandat}")

                            additional_function = assembleeParlementaireInternationale()
                            additional_function_match = assembleeParlementaireInternationale.objects.filter(name__iexact=current_mandat)
                            existing_additional_function = assembleeParlementaireInternationale.objects.all()
                            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                            if additional_function_match.exists() is True:
                                logger.spam(f"AP Internationale exists {current_mandat}")
                                representative.assembleeParlementaireInternationales.add(*additional_function_match)
                                logger.verbose(f"linked function : {representative.assembleeParlementaireInternationales}")
                            else:
                                additional_function.name = current_mandat
                                additional_function.save()
                                representative.assembleeParlementaireInternationales.add(additional_function)
                                logger.verbose(f"commission {representative.assembleeParlementaireInternationales} created and linked")

                    # MISINFO = Mission d’information
                    elif (mandat.get("typeOrgane") == "MISINFO"):
                        # add_m2m_field(current_object,target_model,current_key,target_key)
                        logger.debug(f"mandat {mandat.get('typeOrgane')} {uid}")
                        if mandat.get("dateFin") is None:
                            current_mandat = search_organe_name(mandat.get('organes',{}).get('organeRef',{}))
                            current_mandat = current_mandat[0].strip()
                            logger.spam(f"Mission information name {current_mandat}")

                            additional_function = missionInformation()
                            additional_function_match = missionInformation.objects.filter(name__iexact=current_mandat)
                            existing_additional_function = missionInformation.objects.all()
                            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                            if additional_function_match.exists() is True:
                                logger.spam(f"Mission information exists {current_mandat}")
                                representative.missionInformations.add(*additional_function_match)
                                logger.verbose(f"linked function : {representative.missionInformations}")
                            else:
                                additional_function.name = current_mandat
                                additional_function.save()
                                representative.missionInformations.add(additional_function)
                                logger.verbose(f"commission {representative.missionInformations} created and linked")
                    
                    elif (mandat.get("typeOrgane") == "DELEG") or (mandat.get("typeOrgane") == "DELEGSENAT"):
                        # add_m2m_field(current_object,target_model,current_key,target_key)
                        logger.spam(f"mandat {mandat.get('typeOrgane')} {uid}")
                        if mandat.get("dateFin") is None:
                            current_mandat = search_organe_name(mandat.get('organes',{}).get('organeRef',{}))
                            current_mandat = current_mandat[0].strip()
                            logger.spam(f"Mission information name {current_mandat}")

                            additional_function = delegationAssemblee()
                            additional_function_match = delegationAssemblee.objects.filter(name__iexact=current_mandat)
                            existing_additional_function = delegationAssemblee.objects.all()
                            #print("the commission is" , row["COMMISSION PERMANENTE"] ,"and",exists)
                            if additional_function_match.exists() is True:
                                logger.spam(f"Delegation exists {current_mandat}")
                                representative.delegationAssemblees.add(*additional_function_match)
                                logger.verbose(f"linked function : {representative.delegationAssemblees}")
                            else:
                                additional_function.name = current_mandat
                                additional_function.save()
                                representative.delegationAssemblees.add(additional_function)
                                logger.verbose(f"commission {representative.delegationAssemblees} created and linked")
                    else:
                        if (mandat.get("typeOrgane") == "GOUVERNEMENT") or (mandat.get("typeOrgane") == "MINISTERE")or (mandat.get("typeOrgane") == "PRESREP"):
                            election_department = ""
                            election_region = ""
                            num_circo = ""
                        elif (mandat.get("typeOrgane") == "SENAT"):
                            pass
                        else:
                            raise ValueError(logger.critical(f"organe not handled {mandat.get('typeOrgane',{})} {uid}"))

                # if (election_department == None) or (election_department == ""):
                #     critical_error(f"no location for {uid} {full_name} {election_info} {election_department}")
                if election_department:
                    election_department = get_departement_name(election_department)
                    representative.election_department = election_department
                
                if election_region:
                    representative.election_region = election_region
                
                ### Circonscription
                if 'ministre' in representative_function.lower() :
                    pass
                elif 'dépu' in representative_function.lower() :
                    circo_name = representative.election_department + " circonscription n°" + num_circo
                elif 'sénat' in representative_function.lower() :
                    circo_name = representative.election_department
                
                logger.info(f"circo is {circo_name}")
                if circo_name and circo_name is not None:
                    circo_name_match = Circonscription.objects.filter(name__iexact=circo_name)
                    logger.debug(f"check for {circo_name} is {circo_name_match}")
                    
                    if not circo_name_match.exists():
                        circo = Circonscription(name=circo_name)
                        circo.save()
                        logger.info(f"Circo {circo_name} was created")
                        representative.circonscription = circo 
                    else:
                        logger.info(f"Circo already exists, set it {circo_name_match} ")
                        representative.circonscription = circo_name_match.first()
                else:
                    pass

                #############
                # Parse through "adresses"
                #############
                section_contacts = details.get("adresses", {}).get("adresse", {})
                logger.spam(f"starting contact section")
                if not isinstance(section_contacts, list):
                    section_contacts = [section_contacts]
                for contact in section_contacts:
                    libelle_section = contact.get("typeLibelle")
                    logger.spam(f"libelle section {libelle_section}")
                        
                else:
                    section_contacts = section_contacts
                # try:
                # except KeyError as e:

                # logger.debug(f"section contact {section_contacts}")
                if section_contacts:
                    for contact in section_contacts:
                        libelle_section = contact.get("typeLibelle") or ""
                        logger.spam(f"libelle section {libelle_section}")
                        ### Address Circo
                        if "circonscription" in libelle_section.lower():
                            logger.spam(f"adresse circo {contact}")
                            address = f'{contact.get("numeroRue") or ""} {contact.get("nomRue") or ""}'
                            if contact.get("complementAdresse"):
                                address = f"{address}, {contact.get('complementAdresse')}"
                            representative.address1 = address

                            representative.postal_code = contact.get("codePostal")
                            representative.city  =  contact.get("ville")
                            logger.spam(f"addresse circo {representative.address1}")
                        
                        if "Mèl" in libelle_section:
                            add_email_address(uid,contact,representative)
                            
                        if "Twitter" in libelle_section:
                            add_twitter_account(uid,contact,representative)
                        
                        # tel 
                        if "Téléphone" in libelle_section:
                            add_tel_number(uid,contact,representative)
                        # site web 
                        if "Site internet" in libelle_section:
                            representative.website = contact.get("valElec")
                        # facebook
                        if "Facebook" in libelle_section:
                            representative.facebook = f'https://facebook.com/{contact.get("valElec").replace(" ","")}'
                        # insta
                        if "Instagram" in libelle_section:
                            representative.instagram = f'https://instagram.com/{contact.get("valElec").replace("@","").replace(" ","")}'

                        # linkedin
                        if "Linkedin" in libelle_section:
                            representative.linkedin = f'https://www.linkedin.com{contact.get("valElec")}'
                        
                        
                # haute autorité vie publique - attention ya 2 champs
                
                if "hatvp.fr" in details.get("uri_hatvp"):
                    logger.debug(f"link hatvp {details.get('uri_hatvp')}") 
                    representative.pageHatvp = details.get('uri_hatvp')

                # site Parlement 
                        

                #############
                # Handle postal codes
                #############
                logger.debug(f"start search for postal codes {uid} {representative.function.name}")
                if representative.election_department and 'ministre' not in representative.function.name.lower():
                    
                    # "0" in front of department number is creating matching issues
                    if representative.election_department == "01":
                            dept_key = "1"
                    else:
                        dept_key = representative.election_department

                    if 'sénat' in representative.function.name.lower():
                        representativeCirco = dept_key
                    elif 'déput' in representative.function.name.lower():
                        representativeCirco = representative.circonscription.name.replace(" circonscription n°","-")
                    # print("current circo",representativeCirco,representative['circo'])
                    logger.verbose(f"current circo is {representativeCirco}")
                    
                    # Parse through the Postal Code dict in order to get Codes
                    new_postal_codes = ""
                    for index, values in localityKeys.items():
                        # Circo of senators is whole department, we must match comparison with all columns
                        # logger.info(f"Departement {values[2].lower()}")
                        if 'sénat' in representative.function.name.lower():
                            if representativeCirco.lower() == values[2].lower():
                                logger.spam(f"circo match {representativeCirco} {values[2]} {values[1]}")
                                new_postal_codes +=  values[1]+","
                                logger.spam(f"new postal codes {new_postal_codes}")
                                # if representative.allPostalCodes == "":
                                #     representative.allPostalCodes = values[1]
                                # else:
                                #     representative.allPostalCodes = representative.allPostalCodes + "," +  values[1]
                            
                        else:
                            if representativeCirco.lower() == values[0].lower():
                                logger.spam(f"circo match {representativeCirco} {values[0]}")
                                new_postal_codes +=  values[1]+","
                                # if representative.allPostalCodes == "":
                                #         representative.allPostalCodes = values[1]
                                # else:
                                #     representative.allPostalCodes = representative.allPostalCodes + "," + values[1]
                                

                    if new_postal_codes == "":
                        logger.debug(f"no postal codes {uid}")
                    noDupplicateCodes = []
                    if representative.allPostalCodes == "":
                        originalPostalCodes = new_postal_codes
                    else:
                        originalPostalCodes = representative.allPostalCodes + "," + new_postal_codes

                    logger.spam(f"start postal codes {originalPostalCodes}")
                    originalPostalCodes = originalPostalCodes.split(",")
                    for code in originalPostalCodes:
                        logger.spam(f"code is {code}")
                        code = code.strip()
                        if code not in noDupplicateCodes:
                            noDupplicateCodes.append(code)
                        else: 
                            ValueError("dupplicate",code,noDupplicateCodes)
                    
                    logger.spam(f"No dup CP {noDupplicateCodes}")
                    if  (len(noDupplicateCodes) == 1 and noDupplicateCodes[0].strip() == "") or len(noDupplicateCodes) == 0 :
                        logger.warn(f"{full_name} has no postal codes")
                    else:
                        logger.info(f"Postal codes updated")
                        representative.allPostalCodes = ','.join(noDupplicateCodes)

                
                
                #############
                # Passed Function
                #############
                logger.debug(f"start passed functions")
                get_passed_function(representative,details,uid)

                # Right now we only got the previous functions of the ongoing legislature
                # pass --historical-data flag to upload all previous functions
                if options['historical_data']:
                    ### Once finished, let's look in the previous mandatures for functions
                    get_previous_legislature_functions(representative,uid)

                try:
                    with reversion.create_revision():
                        representative.save()
                        reversion.set_comment("Representative updated via script")
                    count += 1
                    logger.success(f"{count}. {representative.fullname} updated")
                    
                    if options['sleep']:
                        time.sleep(sleepAfterSend)


                except Exception as e:
                    logger.critical(f"error saving {uid} {e}")
                
            if options['dev'] and count >= 9:
                break
                        


                