from django.contrib.sitemaps import Sitemap
from .models import ActionGroup, Representative, ConsultationPublic, AppelSoutien
from reversion.models import Version
from django.conf import settings
from server.settings import *




def get_last_change_date(obj):
    latest_version = Version.objects.get_for_object(obj).first()
    if latest_version:
        return latest_version.revision.date_created
    return None 

def rewrite_front_url(section,urlid):
    print(f"urlid {section} {urlid}")
    if section == 'action':
        path = "/actions/action-group-detail/"
    elif section == 'representative':
        path = "/nos-elus/representative-detail/"
    elif section == 'appel':
        path = "/appel-soutiens/appel-soutiens-detail/"

    elif section == 'consultation':
        path = "/consultations-publique/consultation-detail/"

    elif section == 'law':
        path = "/law-tracker/law-tracker-detail/"
                
    else:
        path = ""

    url = f"{path}@{urlid}@"
    return url

class ActionGroupSitemap(Sitemap):
    changefreq = "daily"
    priority = 1

    def items(self):
        return ActionGroup.objects.exclude(status='pending')

    def lastmod(self, obj):
        return get_last_change_date(obj)

    def location(self, obj):
        section = 'action'
        front_url = rewrite_front_url(section,obj.urlid)
        return front_url

class RepresentativeSitemap(Sitemap):
    changefreq = "monthly"
    priority = 0.8
    def items(self):
        return Representative.objects.all()

    def lastmod(self, obj):
        return get_last_change_date(obj)
    def location(self, obj):
        # Ensure you're using reverse to get the correct URL
        section = 'representative'
        front_url = rewrite_front_url(section,obj.urlid)
        return front_url

class AppelSoutienSitemap(Sitemap):
    changefreq = "weekly"
    priority = 0.8
    def items(self):
        return AppelSoutien.objects.exclude(status='pending')

    def lastmod(self, obj):
        return get_last_change_date(obj)
    
    def location(self, obj):
        # Ensure you're using reverse to get the correct URL
        section = 'appel'
        front_url = rewrite_front_url(section,obj.urlid)
        return front_url

class ConsultationPublicSitemap(Sitemap):
    changefreq = "weekly"
    priority = 1
    def items(self):
        return ConsultationPublic.objects.exclude(status='pending')

    def lastmod(self, obj):
        return get_last_change_date(obj)
        
    def location(self, obj):
        # Ensure you're using reverse to get the correct URL
        section = 'consultation'
        front_url = rewrite_front_url(section,obj.urlid)
        return front_url



## Function to create sitemap index, called in djangoldp_url
class SitemapIndex(Sitemap):
    changefreq = "daily"
    priority = 1
    def items(self):
        PLUGIN_LAW_TRACKER = settings.PLUGIN_LAW_TRACKER
        fields = ['actions', 'representative','appel_soutiens','enquete_public']
        if PLUGIN_LAW_TRACKER:
            fields.append("law_tracker")
        return fields  # List the keys of your individual sitemaps

    def location(self, item):
        return f"/{item}.xml"  # URL structure for individual sitemaps