
from .models import *
from django_q.tasks import async_task
from server.logging_config import setup_logging

### For emailing 
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from django.template import Context, Template

##################
### Newsletter
# Sign up via action forms (interpellation/appel)
##################
def user_newsletter_signup(sender,instance):
    logger.debug(f"action newsletter start {sender} {instance}")
    try:
        senderName = sender.__name__
        logger.debug(f'Newsletter auto-add - Sender is {senderName}')
        if senderName == 'ElectedOfficialContact':
            optinToNewletter = instance.optin2
        elif senderName == 'TextSupporter':
            optinToNewletter = instance.optin2

        logger.info(f'Newsletter check - choice is {optinToNewletter}')
        if optinToNewletter == True:
            newsletter = Newsletter()
            if senderName:
                if senderName == 'ElectedOfficialContact':
                    newsletter.fullname = instance.userName
                    newsletter.userEmail = instance.userEmail
                    newsletter.optin = instance.optin2
                    if instance.linkedAction:
                        newsletter.signedUpFrom = senderName + ': ' + instance.linkedAction.name
                        logger.spam(f'Newsletter linkedactionId is {instance.linkedAction.id}')
                        newsletter.linkedActionId = instance.linkedAction.id
                    else:
                        logger.spam('Newsletter submit with no id')
                        newsletter.signedUpFrom = senderName + ': empty'
                        newsletter.linkedActionId = ''
                        
                    logger.debug(f'Newsletter - add {instance.userName}')

                elif senderName == 'TextSupporter':
                    newsletter.fullname = instance.fullname
                    newsletter.userEmail = instance.email
                    newsletter.optin = instance.optin2
                    if instance.linkedSupport:
                        newsletter.signedUpFrom = senderName + ': ' + instance.linkedSupport.name
                    else:
                        instance.signedUpFrom = senderName + ': 2' 
                    
                    newsletter.linkedActionId = '2'
                    logger.debug(f'Newsletter - add {instance.fullname}')

            else:
                newsletter.fullname = instance.fullname
                newsletter.userEmail = instance.email
                newsletter.optin = instance.optin2
                newsletter.signedUpFrom = 'No sender'
                newsletter.linkedActionId = ''
                logger.debug(f'Newsletter - add {instance.fullname}')

            logger.success(f"{newsletter.fullname} has been added to newsletter")
            newsletter.save()
    except Exception as e:
        logger.error(f"newsletter signup error {instance.fullname} {instance.email}")
        ### TO DO, send email to admin
        # async_task('server.tasks.send_error_email_admin',"invalid_target",instance,group="email_error")


##################
### Email sending to targets from our server
##################

#####
## Confirmation to user
#####
def send_email_confirmation_to_user(instance_pk):
    instance = ElectedOfficialContact.objects.get(pk=instance_pk)

    textTemplate = get_template('email_templates/interpellation-confirmation.txt')
    htmlTemplate = get_template('email_templates/interpellation-confirmation.html')
    
    #Replace backline with <br> to display them in HTML email
    personnalizedContent = instance.content
    # personnalizedContent = personnalizedContent.replace("\r","<br>")
    personnalizedContent = personnalizedContent.replace("\r\n","<br>")
    personnalizedContent = personnalizedContent.replace("\n","<br>")

    #bug - sometimes 'instance.linkedAction' isnt loaded
    if instance.linkedAction:
        emailCC = instance.linkedAction.addCCToEmail
        if emailCC is not None:
            emailCC = emailCC.split(',')
        else:
            emailCC = ''

        ####
        ## If there's a PJ
        ####
        emailPJ = instance.linkedAction.addPJToEmail
        emailPJ_file = ""
        if emailPJ :
            emailPJ_file = emailPJ.path
            # print("Confirmation email - PJ in email :",emailPJ_file)
        else:
            emailPJ_file = None
            print("Confirmation email - No PJ to be added")
    else:
        emailCC = ''
        emailPJ_file = None

    print("cc's are",emailCC)

    d = {"emailSender": {
                "email_destinataires": str(instance.representativeEmails) + emailCC,
                "email_subject": str(instance.subject),
                'email_content':personnalizedContent
            }
        }
    # print('context is',Context(d['emailSender']))
    ### Needed vars
    userEmail = instance.userEmail     
    emailSubject = f'Interpellation  {settings.INSTANCE_NAME} | Récapitulatif de votre envoi'
    ### customize email
    text_content = textTemplate.render(d['emailSender'])
    html_content = htmlTemplate.render(d['emailSender'])
    # print('after context',html_content)
    ### Content processor replaces the <br> by a string (&lt;br&gt;), 
    #let's make it a br again
    html_content = html_content.replace("&lt;br&gt;","<br>")
    ### and just remove it for text version
    text_content = text_content.replace("&lt;br&gt;","\n")

    ### build email
    email = EmailMultiAlternatives( 
        subject=emailSubject,
        body=text_content,
        from_email='"{}" <{}>'.format(settings.INSTANCE_NAME, settings.DEFAULT_FROM_EMAIL),
        to=[userEmail],
    )
    email.attach_alternative(html_content, "text/html")
    if emailPJ_file:
        print('Confirmation email - Adding PJ to email',emailPJ_file)
        email.attach_file(emailPJ_file)
    email.send()

#####
## Update Email counter
#####
def counter_update_emails_sent(instance_pk):
    #in some cases the linkedAction js bugs
    # to avoid issues, don't launch if emptu
    # print('linkedAction',instance.linkedAction)
    try:
        instance = ElectedOfficialContact.objects.get(pk=instance_pk)
        actionGroup = ActionGroup.objects.get(name=instance.linkedAction.name)
        currentCount = actionGroup.representativeEmailSentCounter 
        newCount = (currentCount + 1)
        logger.info("Email sent from our server")
        logger.success(f"### Update Email Counter ### For action : {actionGroup.name}, new count is : {newCount}")
        actionGroup.representativeEmailSentCounter  = newCount
        actionGroup.save()

    except Exception as e:
        # Handle the case where the object does not exist
        logger.critical(f"ActionGroup with the name {e} does not exist")
        logger.error(f"this may be due to an empty 'instance.linkedAction' value : '{instance.linkedAction}'")
        # Additional error handling or logging can be added here
          

##################
### Email/Twitter - Counters on url hit
##################
def process_increment_counter(actionId, counter_field, source, currentDomain):
    try:
        instance = ActionGroup.objects.get(pk=actionId)

        # Increment the specified counter field dynamically
        current_value = getattr(instance, counter_field, None)
        if current_value is None:
            logger.error(f"Counter field {counter_field} does not exist on ActionGroup")
            return

        new_value = current_value + 1
        setattr(instance, counter_field, new_value)
        instance.save(update_fields=[counter_field])

        logger.success(f"### {counter_field.replace('_', ' ').title()} ### {new_value} for Action {instance.name}")

        if source:
            logger.notice(f"Federated Participation from {source}")

    except ActionGroup.DoesNotExist:
        logger.error(f"ActionGroup with ID {actionId} does not exist")
    except Exception as e:
        logger.error(f"Error processing increment for {counter_field}: {e}")

##################
## Appel Soutiens
##################

#####
## Sign up to public model
#####

def signatory_signup_public_model(senderName,instance):
    logger.debug(f"sign up public {senderName} {instance}")
    if senderName == "OrganisationSupporter":
        TextSupporterPublic.objects.create(
            name=instance.organisationName+" signé par "+instance.citizenFullName,  # Adjust fields as needed
            organisation=instance.organisationName,
            linkedSupport=instance.linkedSupport
        )
        logger.success(f'Organisation Signatory {instance.organisationName} added to TextSupporterPublic.')
    else:
        # Create an entry in PublicAppelSupporters if optin3 is True
        TextSupporterPublic.objects.create(
            name=instance.fullname,  # Adjust fields as needed
            organisation=instance.organisation,
            linkedSupport=instance.linkedSupport
        )
        logger.success(f'Individual Signatory {instance.fullname} added to TextSupporterPublic.')

        
    
##################
## Action Group
##################

#####
## Update target counter (total contacts, number of people targeted, days left, ...)
#####
def update_target_counter_actiongroup(instance):
    #get the type of targeting used
    targettingStyle = instance.typeOfTargetting
    actionTargettingTotal = ""
    logger.info("Action stats - Start updating compteur and action info")
    ###
    # Mise à jours des cibles
    ###
    if targettingStyle == 'specificRepresentatives':
        # Update the field value directly without saving the instance
        try:
            ActionGroup.objects.filter(pk=instance.pk).update(
                peopleTargettedCounter=str(instance.specificTargetedRepresentatives.count()) + " décisionnaires ciblés"
            )
            logger.debug(f"Targetted representatives count updated {instance.specificTargetedRepresentatives}")
        except Exception as e:
            logger.critical(f"Targetted representatives count task - failed {instance.specificTargetedRepresentatives}\n Error : {e}")

    ### ciblage par localité
    elif targettingStyle == 'postalCode':
        contentToAdd = []
        for official in instance.targetedOfficials.all():
            logger.debug(f"Linked officials are {official.name} {official.id}")
            logger.debug(f"There are {Representative.objects.filter(function__id=official.id).count()} matching reps")
            contentToAdd.append(str(Representative.objects.filter(function__id=official.id).count())+" "+str(official.name))

        displayContent = ", ".join(contentToAdd)
        
        try:
            ActionGroup.objects.filter(pk=instance.pk).update(
                peopleTargettedCounter=displayContent+" ciblés"
            )
            logger.debug(f"Action Targeted people updated {displayContent}")
        except Exception as e:
            logger.critical(f"Action Targeted people task - failed {displayContent}\n Error : {e}")

    ###
    # Mise à jour de l'affichage du compteur par platforme
    ###
    #get the platforms used
    displayContactsByPlatform = []
    for platform in instance.contactPlateforms.all():
        logger.verbose(f'platform is {platform.name}')
        if 'email' in platform.name.lower():
            if instance.representativeEmailSentCounter != 0:
                displayContactsByPlatform.append(str(instance.representativeEmailSentCounter) + " " + platform.name.lower())
    
        if 'twitter' in platform.name.lower():
            if instance.twitterClickCounter != 0:
                displayContactsByPlatform.append(str(instance.twitterClickCounter) + " " + platform.name.lower().replace('twitter','Twitter'))
        if 'instagram' in platform.name.lower():
            if instance.instagramClickCounter != 0:
                displayContactsByPlatform.append(str(instance.instagramClickCounter) + " " + platform.name.lower().replace('instagram','Instagram'))

    if len(displayContactsByPlatform) != 0:
        try:
            ActionGroup.objects.filter(pk=instance.pk).update(
                displayContactsByPlatform=', '.join(displayContactsByPlatform)
            )
            logger.debug(f"Total by platforms updated {len(displayContactsByPlatform)}")
        except Exception as e:
            logger.critical(f"Update Total by platforms failed {displayContactsByPlatform}\n Error : {e}")


    ###
    # Mise à jour du compteur total
    ###
    numberTweets = instance.twitterClickCounter
    numberEmails = instance.representativeEmailSentCounter
    numberInsta = instance.instagramClickCounter

    totalContacts = numberTweets + numberEmails + numberInsta
    logger.debug(f'Updated Action Stat Update \ntotal contacts are {totalContacts}')
    if totalContacts != 0:
        try:
            ActionGroup.objects.filter(pk=instance.pk).update(
                    totalContacts=totalContacts
                )
            logger.debug(f"Action total updated {totalContacts}")
        except Exception as e:
            logger.critical(f"Action Update task - Total contact failed {totalContacts}\n Error : {e}")

#####
## Target Via Commission permanente
#####
def update_action_target_commission(instance):

    logger.notice(f"Targetting commission : {instance.targetCommission}")
    representatives_in_commission = Representative.objects.filter(permanentCommission__name=instance.targetCommission)

    logger.debug(f"Representatives linked : {representatives_in_commission}")
    if representatives_in_commission.exists():
        ## clear chosen reps, in case previously selected
        instance.specificTargetedRepresentatives.clear()
        ## add matching reps to action
        representatives_in_commission = representatives_in_commission.distinct()
        instance.specificTargetedRepresentatives.set(representatives_in_commission)
        
        # Print the updated value of specificTargetedRepresentatives
        logger.spam(f"Post set: {instance.specificTargetedRepresentatives.all()}")
    else:
        logger.critical(f"No representatives found for commission: {instance.targetCommission}")       

#####
## Target Via Law
#####
def update_action_target_law(instance):
    logger.notice(f"Targetting law : {instance.targetLaw}")

    representatives_in_law = Representative.objects.filter(representative_law__name=instance.targetLaw)

    logger.debug(f"Representatives linked : {representatives_in_law.count()}")
    if representatives_in_law.exists():
        ## clear chosen reps, in case previously selected
        instance.specificTargetedRepresentatives.clear()
        ## add matching reps to action
        representatives_in_law = representatives_in_law.distinct()
        instance.specificTargetedRepresentatives.set(representatives_in_law)
        
        # Print the updated value of specificTargetedRepresentatives
        logger.info(f"Set the following reps : {instance.specificTargetedRepresentatives.all()}")
    else:
        logger.critical(f"No representatives found for commission: {instance.targetCommission}")       

#####
## Update approved email list
#####
def update_approved_emails(instance):
    from risefor_lobbying.models import Email, ApprovedEmail
    debug=False
    debugCountBreak = 10
    emailsInDatabase = Email.objects.all()
    currentlyApprovedEmails = ApprovedEmail.objects.all()
    numberOfApprovedEmails = ApprovedEmail.objects.count()
    listOfApprovedEmails = []
    # create list of approved emails
    for approvedEmail in currentlyApprovedEmails:
        # print(str(approvedEmail))
        if approvedEmail not in listOfApprovedEmails:
            listOfApprovedEmails.append(str(approvedEmail.name))
    
    #create list of emails in database
    count = 0
    for email in emailsInDatabase:
        count +=1
        email = str(email)
        if email not in listOfApprovedEmails:
            approvedEmailModel = ApprovedEmail()
            approvedEmailModel.name = email
            try:
                approvedEmailModel.save()
            except Exception as currentError:
                logger.critical(f"Task - Update approvedEmailModel failed \n{currentError}")
                instance.representativeEmails = currentError
                async_task(f'server.tasks.send_error_email_admin',"Error updating Email",instance,group="approved_email_error")
            
        if (debug == True) and (count >= debugCountBreak):
            break
