#from typing import OrderedDict
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.serializers import ValidationError
from rest_framework.response import Response

from djangoldp.views import LDPViewSet
from risefor_lobbying.models import *
from datetime import date,datetime
from django.conf import settings
from server.logging_config import setup_logging

from urllib.parse import urlparse
import requests

import re

from djangoldp_account.models import LDPUser
from django.views.decorators.cache import never_cache
from django.utils.decorators import method_decorator

class RepresentativeSearchViewSet(LDPViewSet):
    model=Representative
    fields = ['@id','civility','fullname','photo','politicalParty','function','allPostalCodes','directEmails','directPhones','twitterAccount']

###
# Display related representatives
# Or default, return a random list
###
class RelatedRepresentativeViewSet(LDPViewSet):
    model=Representative
    def get_queryset(self, *args, **kwargs):
        # Call the base get_queryset method and apply random ordering
        queryset = super().get_queryset(*args, **kwargs)
        
        return queryset

class financeCampaignViewSet(LDPViewSet):
    model=financeCampaign
    def get_queryset(self, *args, **kwargs):
        querySet = super().get_queryset(*args, **kwargs)
        localActiveCampaign = querySet.filter(status__contains='active')
        return localActiveCampaign

class ActionViewSet(LDPViewSet):
    model = ActionGroup
    

    def get_queryset(self, *args, **kwargs):
        querySet = super().get_queryset(*args, **kwargs)
        localActiveActions = querySet.filter(status__contains='active')
        return localActiveActions
    # def get_queryset(self, *args, **kwargs):
    #     return super().get_queryset(*args, **kwargs).filter(status__contains='active')


from rest_framework.permissions import BasePermission

class AnonymousCanOnlyAdd(BasePermission):
    def has_permission(self, request, view):
        # Allow all users to add (POST)
        if request.method == "POST":
            return True
        # Only allow authenticated users for other methods (GET, DELETE, etc.)
        return request.user.is_authenticated


class newsletterViewSet(LDPViewSet):
    model=Newsletter
    queryset = Newsletter.objects.all()
    permission_classes = [AnonymousCanOnlyAdd]

    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)
    
class ElectedOfficialContactViewSet(LDPViewSet):
    model=ElectedOfficialContact
    queryset = ElectedOfficialContact.objects.all()
    permission_classes = [AnonymousCanOnlyAdd]

    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)
    

class TextSupporterViewSet(LDPViewSet):
    model=TextSupporter
    queryset = TextSupporter.objects.all()
    permission_classes = [AnonymousCanOnlyAdd]

    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)
    

class OrganisationSupporterViewSet(LDPViewSet):
    model=OrganisationSupporter
    queryset = OrganisationSupporter.objects.all()
    permission_classes = [AnonymousCanOnlyAdd]

    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)
    

class ApprovedEmailViewSet(LDPViewSet):
    model=ApprovedEmail
    queryset = ApprovedEmail.objects.all()
    permission_classes = [AnonymousCanOnlyAdd]

    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)
    
class UsersViewSet(LDPViewSet):
    model=LDPUser
    queryset = LDPUser.objects.all()
    permission_classes = [AnonymousCanOnlyAdd]

    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)
    

class AppelSoutienViewSet(LDPViewSet):
    model=AppelSoutien
    def get_queryset(self, *args, **kwargs):
        querySet = super().get_queryset(*args, **kwargs)
        localActiveAppels = querySet.filter(status__contains='active')
        logger.debug(f"local appels are {localActiveAppels}")
        return localActiveAppels

class ConsultationViewSet(LDPViewSet):
    model=ConsultationPublic
    def get_queryset(self, *args, **kwargs):
        querySet = super().get_queryset(*args, **kwargs)
        localActiveConsultation = querySet.filter(status__contains='active')
        return localActiveConsultation

class passedActionViewSet(LDPViewSet):
    model=ActionGroup
    def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(status__contains='finished')


class passedConsultationViewSet(LDPViewSet):
     model=ConsultationPublic
     def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(status__contains='finished')


#view to display total emails
class TotalSubmission(LDPViewSet):
    model = ActionGroup
    def get_queryset(self, *args, **kwargs):
        emailArray = {}
        for action in ActionGroup.objects.all():
            # print('action is',action)
            emailArray[action.name] = action.representativeEmailSentCounter
        return emailArray

    def list(self, request, *args, **kwargs):
        return Response({'Emails envoyé par campagne': self.get_queryset()}) 

def shortLinkRedirect(request, slugs):
    from risefor_lobbying.models import UrlShortner
    from django.shortcuts import redirect
    logger.debug(f"Short like redirect : \nrequested slug : {slugs} \nFull url : {settings.INSTANCE_DOMAINE_NAME}/s/{slugs}")
    
    #search for matching url in shortners
    try:
        data = UrlShortner.objects.get(shortLink=settings.INSTANCE_DOMAINE_NAME+'/s/'+slugs)
        frontUrl = data.url.replace(":","%3A")

        ### Handle "consultation/enquete" section
        if "Lien pour la consultation" in data.actionName:
            #take URL and format it to front end Urls
            finalUrl =  "/consultations-publique/consultation-detail/@"+frontUrl +"@"
        elif "Lien pour l'appel " in data.actionName:
            #take URL and format it to front end Urls
            finalUrl =  "/appel-soutiens/appel-soutiens-detail/@"+frontUrl +"@"
        else:
            finalUrl =  "/actions/action-group-detail/@"+frontUrl +"@"
        logger.success(f"Shortlink redirect to {finalUrl}")
        #redirect
        return redirect(finalUrl)

    #if no match, redirect to actions listing
    except (UrlShortner.DoesNotExist) or None:
        logger.error(f"Short Link Redirection | No match found for {slugs}")
        return redirect("/actions")


import json
from django.http import JsonResponse
from django.views.decorators.http import require_POST
from server.settings import *
from django.views.decorators.csrf import csrf_exempt
from django_q.tasks import async_task
@csrf_exempt
def increment_twitter_click_counter(request):
    return increment_counter_task(request, 'twitterClickCounter')

@csrf_exempt
def increment_email_sent_counter(request):
    return increment_counter_task(request, 'representativeEmailSentCounter')

@csrf_exempt
def increment_instagram_click_counter(request):
    return increment_counter_task(request, 'instagramClickCounter')


def increment_counter_task(request, counter_field):
    try:
        # Load JSON data from the request body
        data = json.loads(request.body)
        actionUrl = data.get('currentUrl')
        source = data.get('source', None)
        logger.debug(f'Increment counter request {actionUrl} {source}')

        # Get the current domain
        currentDomain = settings.INSTANCE_DOMAINE_NAME.replace('http://', '').replace('https://', '')

        # Extract the host from the actionUrl
        actionUrlHost = re.search('action-group-detail\/.*', actionUrl).group()
        actionUrlHost = actionUrlHost.replace('%3A', ':')
        logger.spam(f"Counter host search {actionUrlHost} {currentDomain}")

        # Federation logic: If the action is hosted on another instance
        if currentDomain not in actionUrlHost:
            parts = actionUrlHost.split('@')
            if parts:
                action_source = urlparse(parts[1])
                host_domain = f"{action_source.scheme}://{action_source.netloc}"
                logger.notice(f"Action hosted on another instance {actionUrlHost}")

                # Determine the appropriate URL based on the counter field
                if counter_field == 'twitterClickCounter':
                    post_url = f"{host_domain}/twitter-click-counter/"
                elif counter_field == 'representativeEmailSentCounter':
                    post_url = f"{host_domain}/email-sent-counter/"
                elif counter_field == 'instagramClickCounter':
                    post_url = f"{host_domain}/instagram-click-counter/"
                else:
                    logger.error(f"Unknown counter field {counter_field}")
                    return JsonResponse({'status': 'error', 'message': 'Unknown counter field'}, status=400)

                # Post to the host of the action
                payload = {'currentUrl': actionUrl, 'source': currentDomain}
                response = requests.post(post_url, json=payload)
                logger.info(f"POST request sent to {post_url}, response status code: {response.status_code}")
                return JsonResponse({'status': f'posted to {host_domain}'})

        # Extract the action ID from the URL
        searchForID = re.search(r'(/actiongroups/\d+)', actionUrl)
        if not searchForID:
            logger.critical(f"Counter - No action group in URL given")
            return JsonResponse({'status': 'error'})

        actionId = int(re.search(r'\d+', searchForID.group(1)).group())
        logger.debug(f'Action ID found: {actionId}')

        # Run the task asynchronously
        async_task('risefor_lobbying.tasks.process_increment_counter', actionId, counter_field, source, currentDomain,group="counter_update")

        return JsonResponse({'status': 'success', 'message': 'Processing'}, status=202)

    except json.JSONDecodeError:
        return JsonResponse({'status': 'error', 'message': 'Invalid JSON data'}, status=400)
        
def custom_500_error(request):
    import sys
    from django.shortcuts import render
    exc_type, exc_value, exc_tb = sys.exc_info()
    logger.error(f"Error 500 {exc_value} {exc_type}")
    logger.notice(f"this error in handled in the {os.path.basename(__file__)}")
    logger.notice(f"error detail is' context['error_message']")
    context = {
        'error_message': str(exc_value),
        'error_type': 'Server Error (500)'
    }
    if 'invalid literal for int() with base 10' in context['error_message']:
        return render(request, 'error-pages/int10.html', context, status=500)
    else:
        return render(request, 'error-pages/500.html', context, status=500)

